from import_dbf import models as import_dbf_models
from import_dbf.models import (
    B_A,
    B_B,
    B_E,
    B_L,
    B_N,
    B_U,
    J_H,
    Aut,
    Bib,
    Bib_Desc,
    Dys,
    Ixn,
    Jed,
    Jer,
    Jez,
    Kbn,
    Ldy,
    Lis,
    Loc,
    Poz,
    Pub,
    Ses,
    Usi,
    Wx2,
    Wyd,
)

from django.contrib import admin
from django.contrib.admin.sites import AlreadyRegistered

from bpp.admin.zrodlo import Zrodlo  # noqa


class ImportDbfBaseAdmin(admin.ModelAdmin):
    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def changeform_view(self, request, object_id=None, form_url="", extra_context=None):
        extra_context = extra_context or {}
        extra_context["show_save_and_continue"] = False
        extra_context["show_save"] = False
        return super(ImportDbfBaseAdmin, self).changeform_view(
            request, object_id, extra_context=extra_context
        )


class PozInline(admin.TabularInline):
    model = Poz
    extra = 0


class B_AInline(admin.TabularInline):
    model = B_A
    extra = 0
    autocomplete_fields = ["idt_aut", "idt_jed"]
    fields = ["lp", "idt_aut", "idt_jed", "afiliacja"]


class B_UInline(admin.TabularInline):
    autocomplete_fields = ["idt_usi"]
    fields = ["idt_usi", "comm"]
    model = B_U
    extra = 0


class Bib_DescInline(admin.TabularInline):
    autocomplete_fields = ["idt"]
    fields = ["elem_id", "value", "source"]
    model = Bib_Desc
    ordering = ("elem_id",)
    extra = 0


@admin.register(Bib)
class BibAdmin(ImportDbfBaseAdmin):
    list_display = ["idt", "tytul_or", "title", "zrodlo", "szczegoly", "uwagi"]
    search_fields = ["tytul_or", "title", "zrodlo", "szczegoly", "uwagi", "zrodlo_s"]
    list_filter = ["kbr", "lf", "study_gr", "kwartyl"]
    readonly_fields = ["object_id", "content_type"]
    autocomplete_fields = ["idt2"]
    inlines = [B_AInline, PozInline, B_UInline, Bib_DescInline]


@admin.register(Aut)
class AutAdmin(ImportDbfBaseAdmin):
    list_display = [
        "idt_aut",
        "nazwisko",
        "imiona",
        "tytul",
        # "pokaz_exp_id",
        # "pokaz_ref",
        "orcid_id",
        "stanowisko",
        "prac_od",
        "dat_zwol",
        "kad_nr",
        "idt_jed",
        "tel",
        "email",
        "fg",
        "pbn_id",
        "bpp_autor",
        "bpp_autor_id",
    ]
    search_fields = ["nazwisko", "imiona", "tel", "email"]
    autocomplete_fields = ["idt_jed", "ref", "exp_id"]
    list_filter = ["fg", "tytul"]
    readonly_fields = [
        "bpp_autor",
    ]
    list_select_related = [
        "idt_jed",
        "bpp_autor",
        "bpp_autor__tytul",
    ]

    def pokaz_exp_id(self, obj):
        if obj.exp_id is None:
            return ""
        return str(obj.exp_id) + " (ID: " + str(obj.exp_id_id) + ")"

    def pokaz_ref(self, obj):
        if obj.ref is None:
            return ""
        return str(obj.ref) + " (ID: " + str(obj.ref_id) + ")"


@admin.register(Kbn)
class KbnAdmin(ImportDbfBaseAdmin):
    list_display = ["idt_kbn", "nazwa", "skrot", "to_print", "to_print2", "to_print3"]
    list_filter = ["to_print", "to_print2"]
    readonly_fields = ["bpp_id"]


@admin.register(Jez)
class JezAdmin(ImportDbfBaseAdmin):
    list_display = [
        "nazwa",
        "skrot",
    ]
    readonly_fields = ["bpp_id"]


@admin.register(Jed)
class JedAdmin(ImportDbfBaseAdmin):
    list_display = ["nazwa", "skrot", "wyd_skrot", "email", "www", "to_print"]
    search_fields = ["nazwa", "skrot"]
    list_filter = ["to_print", "wyd_skrot"]
    readonly_fields = ["bpp_jednostka"]


@admin.register(B_A)
class B_AAdmin(ImportDbfBaseAdmin):
    list_display = ["idt", "idt_aut", "idt_jed", "lp"]
    autocomplete_fields = ["idt", "idt_aut", "idt_jed"]
    # list_select_related = ['idt', 'idt_aut', 'idt_jed']
    search_fields = ["idt", "idt__tytul_or"]
    list_filter = ["afiliacja", "tytul", "stanowisko"]


@admin.register(Poz)
class PozAdmin(ImportDbfBaseAdmin):
    list_display = ["idt", "kod_opisu", "lp", "tresc"]
    search_fields = ["idt__tytul_or", "tresc"]
    autocomplete_fields = ["idt"]
    list_select_related = ["idt"]
    list_filter = ["kod_opisu"]


@admin.register(Bib_Desc)
class Bib_DescAdmin(ImportDbfBaseAdmin):
    list_display = ["idt", "elem_id", "value", "source"]
    search_fields = ["idt__tytul_or", "elem_id", "value"]
    autocomplete_fields = ["idt"]
    list_select_related = ["idt"]
    list_filter = ["elem_id"]


@admin.register(B_U)
class BUAdmin(ImportDbfBaseAdmin):
    list_display = ["idt", "idt_usi", "comm"]
    autocomplete_fields = ["idt_usi", "idt"]
    search_fields = ["idt__tytul_or", "idt__pk"]


@admin.register(Usi)
class UsiAdmin(ImportDbfBaseAdmin):
    list_display = ["idt_usi", "usm_f", "usm_sf", "skrot", "nazwa"]
    search_fields = ["idt_usi", "skrot", "nazwa"]
    list_filter = ["usm_f", "usm_sf"]
    autocomplete_fields = ["bpp_id"]


@admin.register(Ses)
class SesAdmin(ImportDbfBaseAdmin):
    list_display = ["redaktor", "file", "login_t", "logout_t"]


@admin.register(Wx2)
class Wx2Admin(ImportDbfBaseAdmin):
    list_display = ["idt_wsx", "skrot", "nazwa", "wsp"]


@admin.register(Ixn)
class IxnAdmin(ImportDbfBaseAdmin):
    list_display = ["idt_pbn", "pbn"]


@admin.register(Wyd)
class WydAdmin(ImportDbfBaseAdmin):
    list_display = ["skrot", "nazwa"]


@admin.register(Ldy)
class LdyAdmin(ImportDbfBaseAdmin):
    list_display = ["id", "dziedzina", "dyscyplina"]


@admin.register(Dys)
class DysAdmin(ImportDbfBaseAdmin):
    list_display = [
        "orcid_id",
        "a_dysc_1",
        "b_dysc_1",
        "c_dysc_1",
        "d_dysc_1",
    ]


@admin.register(B_E)
class B_EAdmin(ImportDbfBaseAdmin):
    list_display = ["idt", "lp", "idt_eng"]


@admin.register(Lis)
class LisAdmin(ImportDbfBaseAdmin):
    list_display = [
        "rok",
        "kategoria",
        "numer",
        "tytul",
        "issn",
        "punkty",
        "sobowtor",
        "errissn",
        "dblissn",
        "dbltitul",
    ]
    list_filter = ["rok", "kategoria", "punkty"]
    search_fields = ["rok", "kategoria", "numer", "tytul", "issn"]


@admin.register(B_L)
class B_LAdmin(ImportDbfBaseAdmin):
    list_display = ["idt", "idt_l"]


@admin.register(J_H)
class J_HAdmin(ImportDbfBaseAdmin):
    list_display = ["idt_jed_f", "idt_jed_t", "rok"]


@admin.register(Pub)
class PubAdmin(ImportDbfBaseAdmin):
    list_display = ["idt_pub", "skrot", "nazwa", "to_print"]
    readonly_fields = ["bpp_id"]


@admin.register(Loc)
class LocAdmin(ImportDbfBaseAdmin):
    list_display = ["ident", "ext"]


@admin.register(Jer)
class JerAdmin(ImportDbfBaseAdmin):
    list_display = ["nr", "od_roku", "skrot", "nazwa", "wyd_skrot"]


@admin.register(B_B)
class B_BAdmin(ImportDbfBaseAdmin):
    autocomplete_fields = ["idt"]
    list_display = ["idt", "lp", "idt_bazy"]


@admin.register(B_N)
class B_NAdmin(ImportDbfBaseAdmin):
    autocomplete_fields = ["idt"]
    list_display = ["idt", "lp", "idt_pbn"]


for elem in import_dbf_models.__all__:
    klass = getattr(import_dbf_models, elem)

    try:

        @admin.register(klass)
        class KlassAdmin(ImportDbfBaseAdmin):
            pass

    except AlreadyRegistered:
        pass



# Generated by Django 2.1.7 on 2019-09-05 18:20

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0177_cache_punktacja_autora_query'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='cache_punktacja_autora_query',
            table='bpp_cache_punktacja_autora',
        ),
    ]



from django.template import Context, Template
from django.test import TestCase
from model_bakery import baker

from bpp.models import Autor, Typ_Odpowiedzialnosci, Wydawnictwo_Ciagle
from bpp.models.system import Jezyk
from bpp.templatetags.prace import strip_at_end, znak_na_koncu
from bpp.tests.tests_legacy.test_reports.util import autor_ciaglego
from bpp.tests.util import any_doktorat, any_jednostka


class TestTemplateTags(TestCase):

    # fixtures = ['typ_odpowiedzialnosci.json', 'charakter_formalny.json',
    #             'tytul.json']

    def setUp(self):
        j = any_jednostka()

        a1 = baker.make(Autor, nazwisko="Kowalski", imiona="Jan", tytul=None, slug="A")
        a2 = baker.make(Autor, nazwisko="Nowak", imiona="Jan", tytul=None, slug="B")
        a3 = baker.make(Autor, nazwisko="Nowak", imiona="Jan", tytul=None, slug="C")

        self.a1 = a1
        self.a2 = a2
        self.a3 = a3

        jezyk = baker.make(Jezyk)
        c = baker.make(
            Wydawnictwo_Ciagle,
            tytul="foo",
            tytul_oryginalny="bar",
            uwagi="fo",
            jezyk=jezyk,
        )
        t, _ign = Typ_Odpowiedzialnosci.objects.get_or_create(
            skrot="aut.", nazwa="autor"
        )
        _ign, _ign = Typ_Odpowiedzialnosci.objects.get_or_create(
            skrot="red.", nazwa="redaktor"
        )
        autor_ciaglego(
            a1, j, c, zapisany_jako="Jan Budnik", typ_odpowiedzialnosci=t, kolejnosc=1
        )
        autor_ciaglego(
            a2,
            j,
            c,
            zapisany_jako="Stefan Kolbe",
            typ_odpowiedzialnosci=Typ_Odpowiedzialnosci.objects.get(skrot="red."),
            kolejnosc=2,
        )
        autor_ciaglego(
            a3,
            j,
            c,
            zapisany_jako="Testowy Autor",
            kolejnosc=-1,
            typ_odpowiedzialnosci=t,
        )

        self.ciagle = c

        self.doktorat = any_doktorat(
            tytul_oryginalny="wtf", tytul="lol", autor=a1, jezyk=jezyk
        )

    def test_autorzy(self):
        self.assertIn(
            "[aut.] Testowy Autor, Jan Budnik, [red.] Stefan Kolbe.".upper(),
            self.ciagle.opis_bibliograficzny(),
        )

    def test_autorzy_doktorat(self):
        self.assertIn("[AUT.] KOWALSKI JAN.", self.doktorat.opis_bibliograficzny())

    def test_autorzy_z_linkami_tekst_przed_po(self):
        self.ciagle.tekst_przed_pierwszym_autorem = "PRZED"
        self.ciagle.tekst_po_ostatnim_autorze = "PO"

        self.assertIn(
            'PRZED [AUT.] <a href="/bpp/autor/C/">Testowy Autor</a>, <a href="/bpp/autor/A/">Jan Budnik</a>, '
            '[RED.] <a href="/bpp/autor/B/">Stefan Kolbe</a>PO.',
            self.ciagle.opis_bibliograficzny(links="normal"),
        )

    def test_autorzy_z_linkami(self):
        self.assertIn(
            '[AUT.] <a href="/bpp/autor/C/">Testowy Autor</a>, <a href="/bpp/autor/A/">Jan Budnik</a>, '
            '[RED.] <a href="/bpp/autor/B/">Stefan Kolbe</a>.',
            self.ciagle.opis_bibliograficzny(links="normal"),
        )

        self.assertIn(
            '[AUT.] <a href="/admin/bpp/autor/%i/change/">Testowy Autor</a>, '
            '<a href="/admin/bpp/autor/%i/change/">Jan Budnik</a>, '
            '[RED.] <a href="/admin/bpp/autor/%i/change/">Stefan Kolbe</a>.'
            % (self.a3.pk, self.a1.pk, self.a2.pk),
            self.ciagle.opis_bibliograficzny(links="admin"),
        )

        self.assertIn(
            '[AUT.] <a href="/bpp/autor/A/">Kowalski Jan</a>.',
            self.doktorat.opis_bibliograficzny(links="normal"),
        )

        self.assertIn(
            '[AUT.] <a href="/admin/bpp/autor/%i/change/">Kowalski Jan</a>.'
            % self.doktorat.autor.pk,
            self.doktorat.opis_bibliograficzny(links="admin"),
        )

    def test_strip_at_end(self):
        self.assertEqual("foo", strip_at_end("foo.,.,.,"))

    def test_znak_na_koncu_alt(self):
        self.assertEqual("foo.", znak_na_koncu("foo.,.,", "."))

        self.assertEqual("", znak_na_koncu(".,.,", "."))

        self.assertEqual(None, znak_na_koncu(None, ",."))

    def test_znak_na_koncu(self):
        template = """
        {% load prace %}
        {{ ciag_znakow|znak_na_koncu:", " }}
        """

        t = Template(template)
        c = Context({"ciag_znakow": "loll..."})
        ret = t.render(c).strip()
        self.assertEqual(ret, "loll,")

    def test_znak_na_poczatku(self):
        template = """
        {% load prace %}
        {{ ciag_znakow|znak_na_poczatku:"," }}
        """

        t = Template(template)
        c = Context({"ciag_znakow": "loll"})
        ret = t.render(c).strip()
        self.assertEqual(ret, ", loll")

    def test_ladne_numery_prac(self):
        template = """
        {% load prace %}
        {{ arr|ladne_numery_prac }}
        """

        t = Template(template)
        c = Context(
            {"arr": {1, 2, 3, 4, 5, 10, 11, 12, 15, 16, 20, 25, 30, 31, 32, 33}}
        )
        ret = t.render(c).strip()
        self.assertEqual(ret, "1-5, 10-12, 15-16, 20, 25, 30-33")

        c = Context({"arr": {1, 3, 4, 5}})
        ret = t.render(c).strip()
        self.assertEqual(ret, "1, 3-5")



from django.db.models import F

from bpp.util import knapsack


def zbieraj_sloty(
    autor_id,
    zadany_slot,
    rok_min,
    rok_max,
    minimalny_pk=None,
    dyscyplina_id=None,
    jednostka_id=None,
    akcja=None,
):
    from bpp.models.cache import Cache_Punktacja_Autora_Query

    rekordy = Cache_Punktacja_Autora_Query.objects.filter(
        rekord__rok__gte=rok_min, rekord__rok__lte=rok_max, autor_id=autor_id
    )
    if dyscyplina_id is not None:
        rekordy = rekordy.filter(dyscyplina_id=dyscyplina_id)

    if jednostka_id is not None:
        rekordy = rekordy.filter(jednostka_id=jednostka_id)

    if minimalny_pk is not None:
        rekordy = rekordy.filter(rekord__punkty_kbn__gte=minimalny_pk)

    res = [
        (name, int(size), int(value))
        for name, size, value in rekordy.values_list(
            "pk", F("slot") * 10000, F("pkdaut") * 10000
        )
    ]  # name, size, value

    id_wpisow_cpaq = [x[0] for x in res]
    sloty = [x[1] for x in res]
    punkty = [x[2] for x in res]

    if akcja == "wszystko":
        return sum(punkty) / 10000, id_wpisow_cpaq, sum(sloty) / 10000

    maks, lista = knapsack(int(zadany_slot * 10000), sloty, punkty, id_wpisow_cpaq)

    sloty_ret = 0
    for elem in lista:
        index = id_wpisow_cpaq.index(elem)
        sloty_ret += sloty[index]

    return maks / 10000, lista, sloty_ret / 10000


def group_emails(group_name):
    "Zwraca maile osób z danej grupy"
    from bpp.models.profile import BppUser

    return (
        BppUser.objects.filter(groups__name=group_name)
        .exclude(email="")
        .exclude(email=None)
        .exclude(email="brak@email.pl")
        .values_list("email", flat=True)
        .distinct()
    )


def editors_emails():
    # E-maile redaktorów -- osób, które mają grupę "wprowadzanie danych"
    from bpp.const import GR_WPROWADZANIE_DANYCH

    return group_emails(GR_WPROWADZANIE_DANYCH)


def zgloszenia_publikacji_emails():
    from bpp.const import GR_ZGLOSZENIA_PUBLIKACJI

    ret = group_emails(GR_ZGLOSZENIA_PUBLIKACJI)
    if ret:
        return ret

    # W sytuacji gdy grupa "zgłoszenia publikacji" nie ma żadnych użytkowników
    # e-mail zostaje wysłany do redaktorów
    return editors_emails()



from .normalization import normalize_exception_reason


class ImproperFileException(Exception):
    pass


class BadNoOfSheetsException(Exception):
    pass


class HeaderNotFoundException(Exception):
    pass


class InputDataError(Exception):
    pass


class XLSParseError(Exception):
    def __init__(self, elem, form, reason):
        self.elem = elem
        self.form = form
        self.reason = reason

    def __str__(self):
        errors = getattr(self.form, "errors")
        if errors:
            errors = errors.as_json()

        return (
            f"Błąd w pliku XLS: "
            f"arkusz nr {self.elem.get('__xls_loc_sheet__')}, "
            f"wiersz nr {self.elem.get('__xls_loc_row__') + 1}, "
            f"etap: {self.reason}, "
            f"błędy: {errors}. "
            f"Oryginalne dane z XLS : {self.elem}"
        )


class XLSMatchError(Exception):
    def __init__(self, elem, object, reason):
        self.elem = elem
        self.object = object
        self.reason = reason

    def __str__(self):
        return (
            f"Nie można zmatchować danych z plikiem XLS: "
            f"arkusz nr {self.elem.get('__xls_loc_sheet__')}, "
            f"wiersz nr {self.elem.get('__xls_loc_row__') + 1}, "
            f"obiekt: {self.object}, "
            f"przyczyna: {self.reason}. "
            f"Oryginalne dane z XLS : {self.elem}"
        )


class BPPDatabaseMismatch(Exception):
    def __init__(self, elem, object, reason):
        self.elem = elem
        self.object = object
        self.reason = reason

    def __str__(self):
        return (
            f"Dane po stronie BPP uniemożliwiają jednoznaczne uzgodnienie rekordu: "
            f"skoroszyt nr {self.elem.get('__xls_loc_sheet__')}, "
            f"wiersz nr {self.elem.get('__xls_loc_row__') + 1}, "
            f"obiekt: {self.object}, "
            f"przyczyna: {self.reason}. "
            f"Oryginalne dane z XLS : {self.elem}"
        )


class BPPDatabaseError(Exception):
    def __init__(self, elem, object, reason):
        self.elem = elem
        self.object = object
        self.reason = reason

    def __str__(self):
        return (
            f"Wystąpił błąd bazy danych po stronie BPP przy próbie integracji danych: "
            f"skoroszyt nr {self.elem.get('__xls_loc_sheet__')}, "
            f"wiersz nr {self.elem.get('__xls_loc_row__') + 1}, "
            f"obiekt: {self.object}, "
            f"przyczyna: {normalize_exception_reason(self.reason)}. "
            f"Oryginalne dane z XLS : {self.elem}"
        )



# Konfiguracja hosta 'master'

import os

from .base import *  # NOQA

DEBUG = True

DEBUG_TOOLBAR = False

SENDFILE_BACKEND = "sendfile.backends.simple"

SELENIUM_DRIVER = "Firefox"


SESSION_COOKIE_SECURE = False
CSRF_COOKIE_SECURE = False
MEDIA_ROOT = os.path.abspath(
    os.path.join(os.path.dirname(__file__), "..", "..", "media")
)
SENDFILE_ROOT = MEDIA_ROOT

CELERY_ALWAYS_EAGER = True
CELERY_EAGER_PROPAGATES_EXCEPTIONS = True

COMPRESS_ENABLED = True
COMPRESS_OFFLINE = False

HTML_MINIFY = False

DATABASES["default"]["CONN_MAX_AGE"] = 0  # noqa

# Vide komentarz w TEMPLATES[0]["OPTIONS"]["loaders"]
CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.dummy.DummyCache",
    }
}

PASSWORD_HASHERS = ["django.contrib.auth.hashers.MD5PasswordHasher"]



"""
Klasy abstrakcyjne
"""
import re
from decimal import Decimal

from django.conf import settings
from django.core.exceptions import ValidationError
from django.core.validators import MinValueValidator, URLValidator
from django.db import models
from django.db.models import CASCADE, SET_NULL, Q, Sum
from django.urls.base import reverse
from taggit.managers import TaggableManager

from django.contrib.postgres.fields import HStoreField
from django.contrib.postgres.search import SearchVectorField as VectorField

from bpp import const
from bpp.fields import DOIField, YearField
from bpp.models.dyscyplina_naukowa import Autor_Dyscyplina, Dyscyplina_Naukowa
from bpp.models.util import ModelZOpisemBibliograficznym, dodaj_autora
from bpp.util import safe_html

ILOSC_ZNAKOW_NA_ARKUSZ = 40000.0


def get_liczba_arkuszy_wydawniczych(liczba_znakow_wydawniczych):
    return round(liczba_znakow_wydawniczych / ILOSC_ZNAKOW_NA_ARKUSZ, 2)


class ModelZeZnakamiWydawniczymi(models.Model):
    liczba_znakow_wydawniczych = models.IntegerField(
        "Liczba znaków wydawniczych", blank=True, null=True, db_index=True
    )

    def ma_wymiar_wydawniczy(self):
        return self.liczba_znakow_wydawniczych is not None

    def wymiar_wydawniczy_w_arkuszach(self):
        return "%.2f" % get_liczba_arkuszy_wydawniczych(self.liczba_znakow_wydawniczych)

    class Meta:
        abstract = True


class ModelZAdnotacjami(models.Model):
    """Zawiera adnotację  dla danego obiektu, czyli informacje, które
    użytkownik może sobie dowolnie uzupełnić.
    """

    ostatnio_zmieniony = models.DateTimeField(auto_now=True, null=True, db_index=True)

    adnotacje = models.TextField(
        help_text="""Pole do użytku wewnętrznego -
        wpisane tu informacje nie są wyświetlane na stronach WWW dostępnych
        dla użytkowników końcowych.""",
        default="",
        blank=True,
        null=False,
        db_index=True,
    )

    class Meta:
        abstract = True


class ModelZPBN_ID(models.Model):
    """Zawiera informacje o PBN_ID"""

    pbn_id = models.IntegerField(
        verbose_name="[Przestarzałe] Identyfikator PBN",
        help_text="[Pole o znaczeniu historycznym] Identyfikator w systemie Polskiej Bibliografii Naukowej (PBN)",
        null=True,
        blank=True,
        unique=True,
        db_index=True,
    )

    class Meta:
        abstract = True


class ModelZNazwa(models.Model):
    """Nazwany model."""

    nazwa = models.CharField(max_length=512, unique=True)

    def __str__(self):
        return self.nazwa

    class Meta:
        abstract = True
        ordering = ["nazwa"]


class NazwaISkrot(ModelZNazwa):
    """Model z nazwą i ze skrótem"""

    skrot = models.CharField(max_length=128, unique=True)

    class Meta:
        abstract = True


class NazwaWDopelniaczu(models.Model):
    nazwa_dopelniacz_field = models.CharField(
        "Nazwa w dopełniaczu", max_length=512, null=True, blank=True
    )

    class Meta:
        abstract = True

    def nazwa_dopelniacz(self):
        if not hasattr(self, "nazwa"):
            return self.nazwa_dopelniacz_field
        if self.nazwa_dopelniacz_field is None or self.nazwa_dopelniacz_field == "":
            return self.nazwa
        return self.nazwa_dopelniacz_field


class ModelZISSN(models.Model):
    """Model z numerem ISSN oraz E-ISSN"""

    issn = models.CharField("ISSN", max_length=32, blank=True, null=True)
    e_issn = models.CharField("e-ISSN", max_length=32, blank=True, null=True)

    class Meta:
        abstract = True


class ModelZISBN(models.Model):
    """Model z numerem ISBN oraz E-ISBN"""

    isbn = models.CharField("ISBN", max_length=64, blank=True, null=True, db_index=True)
    e_isbn = models.CharField(
        "E-ISBN", max_length=64, blank=True, null=True, db_index=True
    )

    class Meta:
        abstract = True


class ModelZInformacjaZ(models.Model):
    """Model zawierający pole 'Informacja z' - czyli od kogo została
    dostarczona informacja o publikacji (np. od autora, od redakcji)."""

    informacja_z = models.ForeignKey(
        "Zrodlo_Informacji", SET_NULL, null=True, blank=True
    )

    class Meta:
        abstract = True


class DwaTytuly(models.Model):
    """Model zawierający dwa tytuły: tytuł oryginalny pracy oraz tytuł
    przetłumaczony."""

    tytul_oryginalny = models.TextField("Tytuł oryginalny", db_index=True)
    tytul = models.TextField("Tytuł", null=True, blank=True, db_index=True)

    def clean(self):
        self.tytul_oryginalny = safe_html(self.tytul_oryginalny)
        self.tytul = safe_html(self.tytul)

    class Meta:
        abstract = True


class ModelZeStatusem(models.Model):
    """Model zawierający pole statusu korekty, oraz informację, czy
    punktacja została zweryfikowana."""

    status_korekty = models.ForeignKey("Status_Korekty", CASCADE)

    class Meta:
        abstract = True


class ModelZAbsolutnymUrl:
    def get_absolute_url(self):
        from django.contrib.contenttypes.models import ContentType

        if hasattr(self, "slug") and self.slug:
            return reverse("bpp:browse_praca_by_slug", args=(self.slug,))

        return reverse(
            "bpp:browse_praca",
            args=(ContentType.objects.get_for_model(self).pk, self.pk),
        )


class ModelZRokiem(models.Model):
    """Model zawierający pole "Rok" """

    rok = YearField(
        help_text="""Rok uwzględniany przy wyszukiwaniu i raportach
        KBN/MNiSW)""",
        db_index=True,
        validators=[MinValueValidator(0)],
    )

    class Meta:
        abstract = True


def nie_zawiera_adresu_doi_org(v):
    if v is None:
        return
    v = v.lower().strip()
    if v.find("doi.org") >= 0:
        raise ValidationError(
            "Pole nie powinno zawierać odnośnika do serwisu doi.org. Identyfikator DOI wpisz "
            "do innego pola. Odnosniki do serwisu DOI, zawierające w swojej tresci numer DOI "
            "są już tworzone przez system. "
        )


class ModelZWWW(models.Model):
    """Model zawierający adres strony WWW"""

    www = models.URLField(
        const.WWW_FIELD_LABEL,
        max_length=1024,
        blank=True,
        null=True,
    )
    dostep_dnia = models.DateField(
        "Dostęp dnia (płatny dostęp)",
        blank=True,
        null=True,
        help_text="""Data dostępu do strony WWW.""",
    )

    public_www = models.URLField(
        const.PUBLIC_WWW_FIELD_LABEL,
        max_length=2048,
        blank=True,
        null=True,
    )
    public_dostep_dnia = models.DateField(
        "Dostęp dnia (wolny dostęp)",
        blank=True,
        null=True,
        help_text="""Data wolnego dostępu do strony WWW.""",
    )

    class Meta:
        abstract = True


class ModelZPubmedID(models.Model):
    pubmed_id = models.BigIntegerField(
        "PubMed ID", blank=True, null=True, help_text="Identyfikator PubMed (PMID)"
    )
    pmc_id = models.CharField("PubMed Central ID", max_length=32, blank=True, null=True)

    class Meta:
        abstract = True


def nie_zawiera_http_https(v):
    if v is None:
        return
    v = v.lower().strip()
    if v.startswith("http") or v.startswith("https") or v.find("doi.org") >= 0:
        raise ValidationError(
            "Pole nie powinno zawierać adresu URL (adresu strony WWW) - wpisz tu wyłącznie "
            "identyfikator DOI; lokalizacją położenia rekordu czyli translacją adresu DOI "
            "na adres URL zajmuje się usługa serwisu doi.org i nie ma potrzeby, aby go "
            "tu wpisywać. "
        )


class ModelZDOI(models.Model):
    doi = DOIField(const.DOI_FIELD_LABEL, null=True, blank=True, db_index=True)

    class Meta:
        abstract = True


class ModelRecenzowany(models.Model):
    """Model zawierający informacje o afiliowaniu/recenzowaniu pracy."""

    recenzowana = models.BooleanField(default=False, db_index=True)

    class Meta:
        abstract = True


IF_MAX_DIGITS = 6
IF_DECIMAL_PLACES = 3


def ImpactFactorField(*args, **kw):
    return models.DecimalField(
        max_digits=IF_MAX_DIGITS,
        decimal_places=IF_DECIMAL_PLACES,
        default=Decimal("0.000"),
        *args,
        **kw,
    )


class ModelPunktowanyBaza(models.Model):
    impact_factor = ImpactFactorField(
        db_index=True,
    )
    punkty_kbn = models.DecimalField(
        "Punkty KBN",
        max_digits=6,
        decimal_places=2,
        default=Decimal("0.00"),
        db_index=True,
    )
    index_copernicus = models.DecimalField(
        "Index Copernicus",
        max_digits=6,
        decimal_places=2,
        default=Decimal("0.00"),
        db_index=True,
    )
    punktacja_wewnetrzna = models.DecimalField(
        "Punktacja wewnętrzna",
        max_digits=6,
        decimal_places=2,
        default=Decimal("0.00"),
        db_index=True,
    )
    punktacja_snip = models.DecimalField(
        "Punktacja SNIP",
        max_digits=6,
        decimal_places=3,
        default=Decimal("0.000"),
        db_index=True,
        help_text="""CiteScore SNIP (Source Normalized Impact per Paper)""",
    )

    kc_impact_factor = models.DecimalField(
        "KC: Impact factor",
        max_digits=6,
        decimal_places=3,
        default=None,
        blank=True,
        null=True,
        help_text="""Jeżeli wpiszesz
        wartość w to pole, to zostanie ona użyta w raporcie dla Komisji
        Centralnej w punkcie IXa tego raportu.""",
        db_index=True,
    )
    kc_punkty_kbn = models.DecimalField(
        "KC: Punkty KBN",
        max_digits=6,
        decimal_places=2,
        default=None,
        blank=True,
        null=True,
        help_text="""Jeżeli wpiszesz
        wartość w to pole, to zostanie ona użyta w raporcie dla Komisji
        Centralnej w punkcie IXa i IXb tego raportu.""",
        db_index=True,
    )
    kc_index_copernicus = models.DecimalField(
        "KC: Index Copernicus",
        max_digits=6,
        decimal_places=2,
        default=None,
        blank=True,
        null=True,
        help_text="""Jeżeli wpiszesz
        wartość w to pole, to zostanie ona użyta w raporcie dla Komisji
        Centralnej w punkcie IXa i IXb tego raportu.""",
    )

    class Meta:
        abstract = True


class ModelPunktowany(ModelPunktowanyBaza):
    """Model zawiereający informację o punktacji."""

    weryfikacja_punktacji = models.BooleanField(default=False)

    class Meta:
        abstract = True

    def ma_punktacje(self):
        """Zwraca 'True', jeżeli ten rekord ma jakąkolwiek punktację,
        czyli jeżeli dowolne z jego pól ma wartość nie-zerową"""

        for pole in POLA_PUNKTACJI:
            f = getattr(self, pole)

            if f is None:
                continue

            if isinstance(f, Decimal):
                if not f.is_zero():
                    return True
            else:
                if f != 0:
                    return True

        return False


POLA_PUNKTACJI = [
    x.name
    for x in ModelPunktowany._meta.fields
    if x.name
    not in [
        "weryfikacja_punktacji",
    ]
]


class ModelTypowany(models.Model):
    """Model zawierający typ KBN oraz język."""

    typ_kbn = models.ForeignKey("Typ_KBN", CASCADE, verbose_name="Typ KBN")
    jezyk = models.ForeignKey("Jezyk", CASCADE, verbose_name="Język")
    jezyk_alt = models.ForeignKey(
        "Jezyk",
        SET_NULL,
        verbose_name="Język alternatywny",
        null=True,
        blank=True,
        related_name="+",
    )

    jezyk_orig = models.ForeignKey(
        "Jezyk",
        SET_NULL,
        verbose_name="Język oryginalny",
        help_text="Dla tłumaczeń. Wartość eksportowana do PBN. ",
        related_name="+",
        null=True,
        blank=True,
    )

    class Meta:
        abstract = True


class ModelZKwartylami(models.Model):
    kwartyl_w_scopus = models.IntegerField(
        "Kwartyl w SCOPUS",
        choices=const.KWARTYLE,
        default=None,
        blank=True,
        null=True,
    )

    kwartyl_w_wos = models.IntegerField(
        "Kwartyl w WoS", choices=const.KWARTYLE, default=None, blank=True, null=True
    )

    class Meta:
        abstract = True


class BazaModeluOdpowiedzialnosciAutorow(models.Model):
    """Bazowa klasa dla odpowiedzialności autorów (czyli dla przypisania
    autora do czegokolwiek innego). Zawiera wszystkie informacje dla autora,
    czyli: powiązanie ForeignKey, jednostkę, rodzaj zapisu nazwiska, ale
    nie zawiera podstawowej informacji, czyli powiązania"""

    autor = models.ForeignKey("bpp.Autor", CASCADE)
    jednostka = models.ForeignKey("bpp.Jednostka", CASCADE)
    kierunek_studiow = models.ForeignKey(
        "bpp.Kierunek_Studiow", SET_NULL, blank=True, null=True
    )
    kolejnosc = models.IntegerField("Kolejność", default=0)
    typ_odpowiedzialnosci = models.ForeignKey(
        "bpp.Typ_Odpowiedzialnosci", CASCADE, verbose_name="Typ odpowiedzialności"
    )
    zapisany_jako = models.CharField(max_length=512)
    afiliuje = models.BooleanField(
        default=True,
        help_text="""Afiliuje
    się do jednostki podanej w przypisaniu. Jednostka nie może być obcą. """,
    )
    zatrudniony = models.BooleanField(
        default=False,
        help_text="""Pracownik
    jednostki podanej w przypisaniu""",
    )

    procent = models.DecimalField(
        "Udział w opracowaniu (procent)",
        max_digits=5,
        decimal_places=2,
        null=True,
        blank=True,
    )

    dyscyplina_naukowa = models.ForeignKey(
        Dyscyplina_Naukowa, on_delete=SET_NULL, null=True, blank=True
    )

    przypieta = models.BooleanField(
        default=True,
        db_index=True,
        help_text="""Możesz odznaczyć, żeby "odpiąć" dyscyplinę od tego autora. Dyscyplina "odpięta" nie będzie
        wykazywana do PBN oraz nie będzie używana do liczenia punktów dla danej pracy.""",
    )

    upowaznienie_pbn = models.BooleanField(
        "Upoważnienie PBN",
        default=False,
        help_text='Tik w polu "upoważnienie PBN" oznacza, że dany autor upoważnił '
        "Uczelnię do sprawozdania tej publikacji w ocenie parametrycznej Uczelni",
    )

    profil_orcid = models.BooleanField(
        "Praca w profilu ORCID autora",
        default=False,
        help_text="Zaznacz, jeżeli praca znajdje się na profilu ORCID autora",
    )

    data_oswiadczenia = models.DateField(
        "Data oświadczenia",
        null=True,
        blank=True,
        help_text="Informacja eksportowana do PBN, gdy uzupełniono",
    )

    class Meta:
        abstract = True
        ordering = ("kolejnosc", "typ_odpowiedzialnosci__skrot")

    def __str__(self):
        return str(self.autor) + " - " + str(self.jednostka.skrot)

    def okresl_dyscypline(self):
        return self.dyscyplina_naukowa

        # Ponizej wykomentowane automatyczne zachowanie, obecne w systemie do wersji 1.0.30-dev2,
        # którego po tej wersji NIE chcemy. Chcemy mieć explicte określoną dyscyplinę naukową.
        # Jednakże, gdyby się okazało, że należy powrócić do jakiejś automatyki w tym temacie,
        # API .okresl_dyscyplinę na ten moment zostaje, jak i resztka z tego kodu, któro
        # zapewniało zachowanie automatyczne:

        # # Jeżeli nie, sprawdź, czy dla danego autora jest określona dyscyplina
        # # na dany rok:
        # try:
        #     ad = Autor_Dyscyplina.objects.get(
        #         autor_id=self.autor_id,
        #         rok=self.rekord.rok,
        #     )
        # except Autor_Dyscyplina.DoesNotExist:
        #     return
        #
        # # Zwróć przypisaną dyscyplinę naukową tylko w sytuacji, gdy jest
        # # określona jedna. Jeżeli są dwie, to nie można określić z automatu
        # if ad.subdyscyplina_naukowa is None:
        #     return ad.dyscyplina_naukowa

    # XXX TODO sprawdzanie, żęby nie było dwóch autorów o tej samej kolejności

    def clean(self):
        # --- Walidacja dyscypliny ---
        # Czy jest określona dyscyplina? Jeżeli tak, to:
        # - rekord nadrzędny musi być określony i mieć jakąś wartość w polu 'Rok'
        # - musi istnieć takie przypisanie autora do dyscypliny dla danego roku
        if self.dyscyplina_naukowa is not None:

            if self.rekord_id is None:
                # Może nie ustalono rekordu nadrzędnego... a moze dodajemy nowy
                # rekord do bazy?

                if self.rekord is None:
                    raise ValidationError(
                        {
                            "dyscyplina_naukowa": "Określono dyscyplinę naukową, ale brak publikacji nadrzędnej. "
                        }
                    )

            if self.rekord is not None and self.rekord.rok is None:
                raise ValidationError(
                    {
                        "dyscyplina_naukowa": "Publikacja nadrzędna nie ma określonego roku."
                    }
                )

            try:
                Autor_Dyscyplina.objects.get(
                    Q(dyscyplina_naukowa=self.dyscyplina_naukowa)
                    | Q(subdyscyplina_naukowa=self.dyscyplina_naukowa),
                    autor=self.autor,
                    rok=self.rekord.rok,
                )
            except Autor_Dyscyplina.DoesNotExist:
                raise ValidationError(
                    {
                        "dyscyplina_naukowa": "Autor nie ma przypisania na dany rok do takiej dyscypliny."
                    }
                )

        # Na tym etapie rekord musi być ustalony:
        ValErrRek = ValidationError(
            "Rekord nadrzędny (pole: rekord) musi być ustalone. "
        )
        if hasattr(self, "rekord"):
            if self.rekord is None:
                raise ValErrRek
        else:
            if self.rekord_id is None:
                raise ValErrRek

        # --- Walidacja procentów ---
        # Znajdź inne obiekty z tego rekordu, które są już w bazie danych, ewentualnie
        # utrudniając ich zapisanie w sytuacji, gdyby ilość procent
        # przekroczyła 100:
        inne = self.__class__.objects.filter(rekord=self.rekord)
        if self.pk:
            inne = inne.exclude(pk=self.pk)
        suma = inne.aggregate(Sum("procent"))["procent__sum"] or Decimal("0.00")
        procent = self.procent or Decimal("0.00")

        if suma + procent > Decimal("100.00"):
            raise ValidationError(
                {
                    "procent": "Suma podanych odpowiedzialności przekracza 100. "
                    "Jeżeli edytujesz rekord, spróbuj zrobić to w dwóch etapach. W pierwszym "
                    "zmniejsz punkty procentowe innym, zapisz, w następnym zwiększ punkty "
                    "procentowe i zapisz ponownie. Rekordy nie zostały zapisane. "
                }
            )

        # --- Walidacja afiliacji ---
        # Jeżeli autor afiliuje na jednostkę która jest obca (skupia_pracownikow=False),
        # to zgłoś błąd

        if (
            self.afiliuje
            and self.jednostka_id is not None
            and self.jednostka.skupia_pracownikow is False
            and getattr(settings, "BPP_WALIDUJ_AFILIACJE_AUTOROW", True)
        ):
            raise ValidationError(
                {
                    "afiliuje": "Jeżeli autor opracował tą pracę w obcej jednostce, to pole "
                    "'Afiliuje' nie powinno być zaznaczone."
                }
            )

    def save(self, *args, **kw):
        if "__disable_bmoa_clean_method" in kw:
            del kw["__disable_bmoa_clean_method"]
        else:
            self.clean()
        from bpp.models import Autor_Jednostka

        if (
            getattr(settings, "BPP_DODAWAJ_JEDNOSTKE_PRZY_ZAPISIE_PRACY", True)
            and not Autor_Jednostka.objects.filter(
                autor_id=self.autor_id, jednostka_id=self.jednostka_id
            ).exists()
        ):
            Autor_Jednostka.objects.create(
                autor_id=self.autor_id,
                jednostka_id=self.jednostka_id,
            )
            # olewamy refresh_from_db i autor.aktualna_jednostka

        return super().save(*args, **kw)


class ModelZeSzczegolami(models.Model):
    """Model zawierający pola: informacje, szczegóły, uwagi, słowa kluczowe."""

    informacje = models.TextField("Informacje", null=True, blank=True, db_index=True)

    szczegoly = models.CharField(
        "Szczegóły", max_length=512, null=True, blank=True, help_text="Np. str. 23-45"
    )

    uwagi = models.TextField(null=True, blank=True, db_index=True)

    slowa_kluczowe = TaggableManager(
        "Słowa kluczowe",
        help_text="Lista słów kluczowych, oddzielonych przecinkiem.",
        blank=True,
    )

    utworzono = models.DateTimeField(
        "Utworzono", auto_now_add=True, blank=True, null=True
    )

    strony = models.CharField(
        max_length=250,
        null=True,
        blank=True,
        help_text="""Jeżeli uzupełnione, to pole będzie eksportowane do
        danych PBN. Jeżeli puste, informacja ta będzie ekstrahowana z
        pola "Szczegóły" w chwili generowania eksportu PBN. Aby uniknąć
        sytuacji, gdy wskutek błędnego wprowadzenia tekstu do pola
        "Szczegóły" informacja ta nie będzie mogła być wyekstrahowana
        z tego pola, kliknij przycisk "Uzupełnij", aby spowodować uzupełnienie
        tego pola na podstawie pola "Szczegóły".
        """,
    )

    tom = models.CharField(
        max_length=50,
        null=True,
        blank=True,
        help_text="""Jeżeli uzupełnione, to pole będzie eksportowane do
        danych PBN. Jeżeli puste, informacja ta będzie ekstrahowana z
        pola 'Informacje'. Kliknięcie przycisku "Uzupełnij" powoduje
        również automatyczne wypełnienie tego pola, o ile do formularza
        zostały wprowadzone odpowiednie informacje. """,
    )

    class Meta:
        abstract = True

    def pierwsza_strona(self):
        return self.strony.split("-")[0]

    def ostatnia_strona(self):
        try:
            return self.strony.split("-")[1]
        except IndexError:
            return self.pierwsza_strona()


class ModelZNumeremZeszytu(models.Model):
    nr_zeszytu = models.CharField(
        max_length=50,
        null=True,
        blank=True,
        help_text="""Jeżeli uzupełnione, to pole będzie eksportowane do
        danych PBN. Jeżeli puste, informacja ta będzie ekstrahowana z
        pola 'Informacje'. Kliknięcie przycisku "Uzupełnij" powoduje
        również automatyczne wypełnienie tego pola, o ile do formularza
        zostały wprowadzone odpowiednie informacje. """,
    )

    class Meta:
        abstract = True


class ModelZCharakterem(models.Model):
    charakter_formalny = models.ForeignKey(
        "bpp.Charakter_Formalny", CASCADE, verbose_name="Charakter formalny"
    )

    class Meta:
        abstract = True


class ModelPrzeszukiwalny(models.Model):
    """Model zawierający pole pełnotekstowego przeszukiwania
    'search_index'"""

    search_index = VectorField()
    tytul_oryginalny_sort = models.TextField(db_index=True, default="")

    class Meta:
        abstract = True


class ModelZLegacyData(models.Model):
    """Model zawierający informacje zaimportowane z poprzedniego systemu,
    nie mające odpowiednika w nowych danych, jednakże pozostawione na
    rekordzie w taki sposób, aby w razie potrzeby w przyszłości można było
    z nich skorzystać"""

    legacy_data = HStoreField(blank=True, null=True)

    class Meta:
        abstract = True


class RekordBPPBaza(
    ModelZPBN_ID, ModelZOpisemBibliograficznym, ModelPrzeszukiwalny, ModelZLegacyData
):
    """Klasa bazowa wszystkich rekordów (patenty, prace doktorskie,
    habilitacyjne, wydawnictwa zwarte i ciągłe)"""

    class Meta:
        abstract = True


class ModelWybitny(models.Model):
    praca_wybitna = models.BooleanField(default=False)
    uzasadnienie_wybitnosci = models.TextField(
        "Uzasadnienie wybitności", default="", blank=True
    )

    class Meta:
        abstract = True


class LinkDoPBNMixin:
    url_do_pbn = None
    atrybut_dla_url_do_pbn = "pbn_uid_id"

    def link_do_pbn_wartosc_id(self):
        return getattr(self, self.atrybut_dla_url_do_pbn)

    def link_do_pbn(self):
        assert self.url_do_pbn, "Określ parametr self.url_do_pbn"

        from bpp.models import Uczelnia

        uczelnia = Uczelnia.objects.get_default()
        if uczelnia is not None:
            return self.url_do_pbn.format(
                pbn_api_root=uczelnia.pbn_api_root,
                pbn_uid_id=self.link_do_pbn_wartosc_id(),
            )


class ModelZPBN_UID(LinkDoPBNMixin, models.Model):
    pbn_uid = models.ForeignKey(
        "pbn_api.Publication",
        verbose_name=const.PBN_UID_FIELD_LABEL,
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
    )

    url_do_pbn = const.LINK_PBN_DO_PUBLIKACJI

    class Meta:
        abstract = True


class ModelZOplataZaPublikacje(models.Model):
    opl_pub_cost_free = models.BooleanField(
        verbose_name="Publikacja bezkosztowa", null=True
    )
    opl_pub_research_potential = models.BooleanField(
        verbose_name="Środki finansowe art. 365 pkt 2 ustawy",
        null=True,
        help_text="Środki finansowe, o których mowa w art. 365 pkt 2 ustawy",
    )
    opl_pub_research_or_development_projects = models.BooleanField(
        verbose_name="Środki finansowe na realizację projektu",
        null=True,
        help_text="Środki finansowe przyznane na realizację projektu "
        "w zakresie badań naukowych lub prac rozwojowych",
    )

    opl_pub_other = models.BooleanField(
        verbose_name="Inne środki finansowe", null=True, blank=True, default=None
    )

    opl_pub_amount = models.DecimalField(
        max_digits=20,
        decimal_places=2,
        verbose_name="Kwota brutto (zł)",
        null=True,
        blank=True,
    )

    class Meta:
        abstract = True

    def clean(self):
        if self.opl_pub_cost_free:
            # Publikacja bezkosztowa...

            if self.opl_pub_amount is not None and self.opl_pub_amount > 0:
                # ... musi mieć kwotę za publikację równą zero
                raise ValidationError(
                    {
                        "opl_pub_amount": "Publikacja bezkosztowa, ale kwota opłaty za publikację większa od zera."
                        "Proszę o skorygowanie"
                    }
                )

            if (
                self.opl_pub_research_potential
                or self.opl_pub_research_or_development_projects
                or self.opl_pub_other
            ):
                # ... oraz odznaczone pozostałe pola

                errmsg = """Jeżeli zaznaczono publikację jako bezkosztową, to pozostałe pola dotyczące
                środków finansowych nie mogą być zaznaczone na 'TAK', a koszt powinien być równy 0.00 zł. Przejrzyj
                te pola i odznacz je. """

                errmsg2 = """Pole nie może być zaznaczone na 'TAK' dla publikacji bezkosztowej. """

                errdct = {
                    "opl_pub_cost_free": errmsg,
                }

                if self.opl_pub_research_potential:
                    errdct["opl_pub_research_potential"] = errmsg2

                if self.opl_pub_research_or_development_projects:
                    errdct["opl_pub_research_or_development_projects"] = errmsg2

                if self.opl_pub_other:
                    errdct["opl_pub_other"] = errmsg2

                raise ValidationError(errdct)
        else:
            # Publikacja kosztowa z kolei (self.opl_pub_cost_free jest None
            # albo False) ...
            if self.opl_pub_amount is not None and self.opl_pub_amount > 0:
                # ...jeżeli ma wpisany koszt, musi miec zaznaczony któreś z pól:
                if (
                    not self.opl_pub_research_or_development_projects
                    and not self.opl_pub_research_potential
                    and not self.opl_pub_other
                ):

                    errmsg = (
                        "Jeżeli wpisano opłatę za publikację, należy dodatkowo zaznaczyć, z jakich środków"
                        " została ta opłata zrealizowana. Przejrzyj pola dotyczące środków finansowych "
                        "i ustaw wartość na 'TAK' przynajmniej w jednym z nich - np w tym ... "
                    )

                    errmsg2 = "... lub w tym ..."
                    errmsg3 = "... lub tutaj. "

                    raise ValidationError(
                        {
                            "opl_pub_research_potential": errmsg,
                            "opl_pub_research_or_development_projects": errmsg2,
                            "opl_pub_other": errmsg3,
                        }
                    )

            else:
                # ... jeżeli nie ma wpisanego kosztu a ma zaznaczone któreś z pól to też źle
                if (
                    self.opl_pub_research_or_development_projects
                    or self.opl_pub_research_potential
                    or self.opl_pub_other
                ):
                    errdct = {"opl_pub_amount": "Tu należy uzupełnić kwotę. "}

                    errmsg = (
                        "Jeżeli wybrano pola dotyczące opłaty za publikację, należy dodatkowo wpisać kwotę... "
                        "lub od-znaczyć te pola. "
                    )

                    if self.opl_pub_research_potential:
                        errdct["opl_pub_research_potential"] = errmsg

                    if self.opl_pub_research_or_development_projects:
                        errdct["opl_pub_research_or_development_projects"] = errmsg

                    if self.opl_pub_other:
                        errdct["opl_pub_other"] = errmsg

                    raise ValidationError(errdct)
                else:
                    if self.opl_pub_cost_free is not None:
                        # jeżeli nie ma wpisanego kosztu i nie ma zaznaczonego
                        # zadnego z pól to tym bardziej źle
                        errdct = {"opl_pub_amount": "Tu należy uzupełnić kwotę. "}

                        errmsg = (
                            "Jeżeli publikacja nie była bezkosztowa, należy zaznaczyć "
                            "przynajmniej jedno z tych pól"
                        )

                        errdct["opl_pub_research_potential"] = errmsg
                        errdct["opl_pub_research_or_development_projects"] = errmsg
                        errdct["opl_pub_other"] = errmsg

                        raise ValidationError(errdct)


class Wydawnictwo_Baza(RekordBPPBaza):
    """Klasa bazowa wydawnictw (prace doktorskie, habilitacyjne, wydawnictwa
    ciągłe, zwarte -- bez patentów)."""

    def __str__(self):
        return self.tytul_oryginalny

    class Meta:
        abstract = True


url_validator = URLValidator()


strony_regex = re.compile(
    r"(?P<parametr>s{1,2}\.)\s*"
    r"(?P<poczatek>(\w*\d+|\w+|\d+))"
    r"((-)(?P<koniec>(\w*\d+|\w+|\d+))|)",
    flags=re.IGNORECASE,
)

alt_strony_regex = re.compile(
    r"(?P<poczatek>\d+)(-(?P<koniec>\d+)|)(\s*s.|)", flags=re.IGNORECASE
)

BRAK_PAGINACJI = ("[b. pag.]", "[b.pag.]", "[b. pag]", "[b. bag.]")


def wez_zakres_stron(szczegoly):
    """Funkcja wycinająca informacje o stronach z pola 'Szczegóły'"""
    if not szczegoly:
        return

    for bp in BRAK_PAGINACJI:
        if szczegoly.find(bp) >= 0:
            return "brak"

    def ret(res):
        d = res.groupdict()
        if "poczatek" in d and "koniec" in d and d["koniec"] is not None:
            return "{}-{}".format(d["poczatek"], d["koniec"])

        return "%s" % d["poczatek"]

    res = strony_regex.search(szczegoly)
    if res is not None:
        return ret(res)

    res = alt_strony_regex.search(szczegoly)
    if res is not None:
        return ret(res)


parsed_informacje_regex = re.compile(
    r"(\[online\])?\s*"
    r"(?P<rok>\d\d+)"
    r"(\s*(vol|t|r|bd)\.*\s*\[?(?P<tom>[A-Za-z]?\d+)\]?)?"
    # To poniżej to była kiedyś jedna długa linia
    r"(\s*(iss|nr|z|h|no)?\.*\s*(?P<numer>((\d+\w*([\/-]\d*\w*)?)\s*((e-)?(suppl|supl)?\.?(\s*\d+|\w+)?)|"
    r"((e-)?(suppl|supl)?\.?\s*\d+(\/\d+)?)|(\d+\w*([\/-]\d*\w*)?))|\[?(suppl|supl)\.\]?))?",
    flags=re.IGNORECASE,
)


def parse_informacje_as_dict(
    informacje, parsed_informacje_regex=parsed_informacje_regex
):
    """Wycina z pola informacje informację o tomie lub numerze lub roku.

    Jeśli mamy zapis "Vol.60 supl.3" - to "supl.3";
    jeśli mamy zapis "Vol.61 no.2 suppl.2" - to optymalnie byłoby, żeby do pola numeru trafiało "2 suppl.2",
    jeśli zapis jest "Vol.15 no.5 suppl." - "5 suppl."
    """
    if not informacje:
        return {}

    p = parsed_informacje_regex.search(informacje)
    if p is not None:
        return p.groupdict()
    return {}


def parse_informacje(informacje, key):
    "Wstecznie kompatybilna wersja funkcji parse_informacje_as_dict"
    return parse_informacje_as_dict(informacje).get(key)


class ModelZSeria_Wydawnicza(models.Model):
    seria_wydawnicza = models.ForeignKey(
        "bpp.Seria_Wydawnicza", CASCADE, blank=True, null=True
    )

    numer_w_serii = models.CharField(max_length=512, blank=True, null=True)

    class Meta:
        abstract = True


class ModelZKonferencja(models.Model):
    konferencja = models.ForeignKey("bpp.Konferencja", CASCADE, blank=True, null=True)

    class Meta:
        abstract = True


class ModelZOpenAccess(models.Model):
    openaccess_wersja_tekstu = models.ForeignKey(
        "Wersja_Tekstu_OpenAccess",
        CASCADE,
        verbose_name="OpenAccess: wersja tekstu",
        blank=True,
        null=True,
    )

    openaccess_licencja = models.ForeignKey(
        "Licencja_OpenAccess",
        CASCADE,
        verbose_name="OpenAccess: licencja",
        blank=True,
        null=True,
    )

    openaccess_czas_publikacji = models.ForeignKey(
        "Czas_Udostepnienia_OpenAccess",
        CASCADE,
        verbose_name="OpenAccess: czas udostępnienia",
        blank=True,
        null=True,
    )

    openaccess_ilosc_miesiecy = models.PositiveIntegerField(
        "OpenAccess: ilość miesięcy",
        blank=True,
        null=True,
        help_text="Ilość miesięcy jakie upłynęły od momentu opublikowania do momentu udostępnienia",
    )

    class Meta:
        abstract = True


class ModelZLiczbaCytowan(models.Model):
    liczba_cytowan = models.PositiveIntegerField(
        verbose_name="Liczba cytowań",
        null=True,
        blank=True,
        help_text="""Wartość aktualizowana jest automatycznie raz na kilka dni w przypadku
        skonfigurowania dostępu do API WOS AMR (przez obiekt 'Uczelnia'). Możesz również
        czaktualizować tą wartość ręcznie, naciskając przycisk. """,
    )

    class Meta:
        abstract = True


class ModelZMiejscemPrzechowywania(models.Model):
    numer_odbitki = models.CharField(max_length=50, null=True, blank=True)

    class Meta:
        abstract = True


class MaProcentyMixin:
    def ma_procenty(self):
        for autor in self.autorzy_set.all():
            if autor.procent:
                return True
        return False


class NieMaProcentowMixin:
    def ma_procenty(self):
        return False


class DodajAutoraMixin:
    """Funkcja pomocnicza z dodawaniem autora do rekordu, raczej na 99%
    używana tylko i wyłącznie przez testy. Musisz określić self.autor_rekordu_class
    czyli np dla Wydawnictwo_Zwarte ta zmienna powinna przyjąć wartość
    Wydawnictwo_Zwarte_Autor."""

    autor_rekordu_klass = None

    def dodaj_autora(
        self,
        autor,
        jednostka,
        zapisany_jako=None,
        typ_odpowiedzialnosci_skrot="aut.",
        kolejnosc=None,
        dyscyplina_naukowa=None,
        afiliuje=True,
    ):
        """
        :rtype: bpp.models.abstract.BazaModeluOdpowiedzialnosciAutorow
        """
        return dodaj_autora(
            klass=self.autor_rekordu_klass,
            rekord=self,
            autor=autor,
            jednostka=jednostka,
            zapisany_jako=zapisany_jako,
            typ_odpowiedzialnosci_skrot=typ_odpowiedzialnosci_skrot,
            kolejnosc=kolejnosc,
            dyscyplina_naukowa=dyscyplina_naukowa,
            afiliuje=afiliuje,
        )


class ModelOpcjonalnieNieEksportowanyDoAPI(models.Model):
    nie_eksportuj_przez_api = models.BooleanField(
        "Nie eksportuj przez API",
        default=False,
        db_index=True,
        help_text="Jeżeli zaznaczone, to ten rekord nie będzie dostępny przez JSON REST API",
    )

    class Meta:
        abstract = True


class ModelZPrzeliczaniemDyscyplin(models.Model):
    def przelicz_punkty_dyscyplin(self):
        from bpp.models.sloty.core import IPunktacjaCacher
        from bpp.models.uczelnia import Uczelnia

        ipc = IPunktacjaCacher(self, Uczelnia.objects.get_default())
        ipc.removeEntries()
        if ipc.canAdapt():
            ipc.rebuildEntries()
        return ipc.serialize()

    def odpiete_dyscypliny(self):
        return self.autorzy_set.exclude(dyscyplina_naukowa=None).exclude(przypieta=True)

    class Meta:
        abstract = True


class BazaModeluStreszczen(models.Model):
    jezyk_streszczenia = models.ForeignKey(
        "bpp.Jezyk", null=True, blank=True, on_delete=models.SET_NULL
    )
    streszczenie = models.TextField(blank=True, null=True)

    class Meta:
        abstract = True



# Generated by Django 3.0.11 on 2021-03-15 19:30

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0248_autorzy_mat_idx"),
    ]

    operations = [
        migrations.CreateModel(
            name="Dyscyplina_Zrodla",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "dyscyplina",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="bpp.Dyscyplina_Naukowa",
                    ),
                ),
                (
                    "zrodlo",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="bpp.Zrodlo"
                    ),
                ),
            ],
            options={
                "verbose_name": "przypisanie dyscypliny do źródła",
                "verbose_name_plural": "przypisania dyscyplin do źródeł",
                "ordering": ["zrodlo__nazwa", "dyscyplina__nazwa"],
                "unique_together": {("zrodlo", "dyscyplina")},
            },
        ),
    ]



# Create your views here.

from braces.views import GroupRequiredMixin
from django.http import HttpResponseRedirect

from import_pracownikow.forms import NowyImportForm
from import_pracownikow.models import ImportPracownikow
from long_running.views import (
    CreateLongRunningOperationView,
    LongRunningDetailsView,
    LongRunningOperationsView,
    LongRunningResultsView,
    LongRunningRouterView,
    RestartLongRunningOperationView,
)

from django.contrib import messages

from bpp.models import Uczelnia


class BaseImportPracownikowMixin(GroupRequiredMixin):
    group_required = "wprowadzanie danych"
    model = ImportPracownikow


class ListaImportowView(BaseImportPracownikowMixin, LongRunningOperationsView):
    pass


class NowyImportView(BaseImportPracownikowMixin, CreateLongRunningOperationView):
    form_class = NowyImportForm


class ImportPracownikowRouterView(BaseImportPracownikowMixin, LongRunningRouterView):
    redirect_prefix = "import_pracownikow:import_pracownikow"


class ImportPracownikowDetailsView(BaseImportPracownikowMixin, LongRunningDetailsView):
    pass


class ImportPracownikowResultsView(BaseImportPracownikowMixin, LongRunningResultsView):
    def autorzy_spoza_pliku(self):
        uczelnia = Uczelnia.objects.get_for_request(self.request)
        return self.parent_object.autorzy_spoza_pliku_set(
            uczelnia=uczelnia
        ).select_related("autor", "autor__tytul", "jednostka", "jednostka__wydzial")

    def get_context_data(self, **kwargs):
        return super().get_context_data(autorzy_spoza_pliku=self.autorzy_spoza_pliku())


class ImportPracownikowResetujPodstawoweMiejscePracyView(ImportPracownikowResultsView):
    def get(self, request, *args, **kwargs):
        uczelnia = Uczelnia.objects.get_for_request(self.request)

        self.parent_object.odepnij_autorow_spoza_pliku(uczelnia=uczelnia)

        messages.info(
            request, "Podstawowe miejsca pracy autorów zostały zaktualizowane."
        )
        return HttpResponseRedirect("..")


class RestartImportView(BaseImportPracownikowMixin, RestartLongRunningOperationView):
    pass



from rest_framework import serializers

from api_v1.serializers.util import ChoicesSerializerField
from bpp.models import Zrodlo, Rodzaj_Zrodla, Wydawca
from bpp.models.wydawca import Poziom_Wydawcy


class Poziom_WydawcySerializer(serializers.HyperlinkedModelSerializer):
    poziom = ChoicesSerializerField()
    wydawca = serializers.HyperlinkedRelatedField(
        view_name="api_v1:wydawca-detail", read_only=True
    )

    class Meta:
        model = Poziom_Wydawcy
        fields = ["id", "wydawca", "rok", "poziom"]


class WydawcaSerializer(serializers.HyperlinkedModelSerializer):
    alias_dla = serializers.HyperlinkedRelatedField(
        view_name="api_v1:wydawca-detail", read_only=True
    )

    poziom_wydawcy_set = serializers.HyperlinkedRelatedField(
        many=True, read_only=True, view_name="api_v1:poziom_wydawcy-detail"
    )

    class Meta:
        model = Wydawca
        fields = ["id", "nazwa", "alias_dla", "poziom_wydawcy_set"]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-11-10 23:36
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0113_auto_20171111_0035'),
    ]

    operations = [
        migrations.RenameField(
            model_name='autor',
            old_name='pokazuj_na_stronach_jednostek',
            new_name='pokazuj',
        ),
    ]



# Generated by Django 3.0.11 on 2021-03-07 13:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0242_auto_20210307_1110"),
    ]

    operations = [
        migrations.AlterField(
            model_name="autor",
            name="system_kadrowy_id",
            field=models.PositiveIntegerField(
                blank=True,
                db_index=True,
                help_text="Identyfikator cyfrowy, używany do matchowania autora z danymi z systemu kadrowego Uczelni",
                null=True,
                unique=True,
                verbose_name="Identyfikator w systemie kadrowym",
            ),
        ),
    ]



from datetime import timedelta

import pytest
from django.urls import reverse
from model_bakery import baker

from django.utils.timezone import localtime

from bpp.models import Patent


@pytest.mark.django_db
def test_rest_api_patent_detail(client, patent):
    res = client.get(reverse("api_v1:patent-detail", args=(patent.pk,)))
    assert res.status_code == 200


@pytest.mark.django_db
def test_rest_api_patent_list(client, patent):
    res = client.get(reverse("api_v1:patent-list"))
    assert res.status_code == 200


@pytest.mark.django_db
def test_rest_api_patent_filtering_1(api_client, patent):
    czas = localtime(patent.ostatnio_zmieniony).strftime("%Y-%m-%d %H:%M:%S")

    res = api_client.get(
        reverse("api_v1:patent-list") + f"?ostatnio_zmieniony_after={czas}"
    )
    assert res.json()["count"] == 1


@pytest.mark.django_db
def test_rest_api_patent_filtering_2(api_client, patent):
    czas = localtime(patent.ostatnio_zmieniony + timedelta(seconds=1)).strftime(
        "%Y-%m-%d %H:%M:%S"
    )

    res = api_client.get(
        reverse("api_v1:patent-list") + f"?ostatnio_zmieniony_after={czas}"
    )
    assert res.json()["count"] == 0


@pytest.mark.django_db
def test_rest_api_patent_filtering_rok(api_client, wydawnictwo_ciagle, rok):
    res = api_client.get(reverse("api_v1:patent-list") + f"?rok_min={rok+1}")
    assert res.json()["count"] == 0


@pytest.mark.django_db
def test_rest_api_patent_ukryj_status(
    api_client, patent, uczelnia, przed_korekta, po_korekcie
):

    res = api_client.get(reverse("api_v1:patent-list"))
    assert res.json()["count"] == 1

    patent.status_korekty = przed_korekta
    patent.save()

    uczelnia.ukryj_status_korekty_set.create(status_korekty=przed_korekta)
    res = api_client.get(reverse("api_v1:patent-list"))
    assert res.json()["count"] == 0


@pytest.fixture
def wiele_patentow(db, jezyki, charaktery_formalne):
    for a in range(100):
        baker.make(Patent)


@pytest.mark.django_db
def test_rest_api_patent_no_queries(
    wiele_patentow, django_assert_max_num_queries, api_client
):
    with django_assert_max_num_queries(11):
        api_client.get(reverse("api_v1:patent-list"))



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-19 20:38
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0093_konferencja_wydawnictwo_ciagle'),
    ]

    operations = [
        migrations.CreateModel(
            name='Seria_Wydawnicza',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('nazwa', models.CharField(max_length=512, unique=True)),
                ('issn', models.CharField(blank=True, max_length=32, null=True, verbose_name='ISSN')),
                ('e_issn', models.CharField(blank=True, max_length=32, null=True, verbose_name='e-ISSN')),
            ],
            options={
                'ordering': ['nazwa'],
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='wydawnictwo_zwarte',
            name='numer_w_serii',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='wydawnictwo_zwarte',
            name='seria_wydawnicza',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='bpp.Seria_Wydawnicza'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-06-12 21:04
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0134_liczba_cytowan_wyszukaj'),
        ('bpp', '0134_uczelnia_domyslnie_afiliuje'),
    ]

    operations = [
    ]



# -*- encoding: utf-8 -*-
from crispy_forms.helper import FormHelper
from crispy_forms_foundation.layout import Layout, Fieldset, ButtonHolder, \
    Submit, Row, Column
from password_policies.forms import PasswordPoliciesChangeForm

class BppPasswordChangeForm(PasswordPoliciesChangeForm):
    def __init__(self, *args, **kw):

        self.helper = FormHelper()
        self.helper.form_method = 'post'
        self.helper.layout = Layout(
            Fieldset(
                'Zmień hasło',
                Row(Column('old_password')),
                Row(Column('new_password1')),
                Row(Column('new_password2')),
            ),
            ButtonHolder(
                Submit('submit', 'Zmień hasło', css_class='button white')
            )
        )

        super(BppPasswordChangeForm, self).__init__(*args, **kw)



import pytest

from crossref_bpp.forms import PobierzZCrossrefAPIForm
from crossref_bpp.models import CrossrefAPICache


@pytest.mark.vcr
@pytest.mark.django_db
def test_PobierzZCrossrefAPIFrom_clean():
    assert CrossrefAPICache.objects.count() == 0
    f = PobierzZCrossrefAPIForm({"identyfikator_doi": "10.1080/14786419.2014.955494"})
    assert f.is_valid()
    assert CrossrefAPICache.objects.count() == 1



import sys


def qual(clazz):
    """
    Return full import path of a class.

    Function stolen from twisted.python.reflect!
    """
    return clazz.__module__ + "." + clazz.__name__


def str_to_class(path):
    """Return a class from a string representation"""
    split = path.split(".")
    module = ".".join(split[:-1])
    field = split[-1]

    try:
        identifier = getattr(sys.modules[module], field)
    except AttributeError:
        raise NameError("%s doesn't exist." % field)
    return identifier



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-19 11:01
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0092_pokazuj_status_korekty'),
    ]

    operations = [
        migrations.AddField(
            model_name='wydawnictwo_ciagle',
            name='konferencja',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='bpp.Konferencja'),
        ),
    ]



import time

import pytest
from django.urls import reverse
from selenium.common.exceptions import NoAlertPresentException
from selenium.webdriver.common.keys import Keys

from fixtures import pbn_pageable_json, pbn_publication_json
from pbn_api.client import PBN_GET_PUBLICATION_BY_ID_URL, PBN_SEARCH_PUBLICATIONS_URL
from pbn_api.models import Publication
from pbn_api.models.publication import STATUS_ACTIVE

from bpp.tests import (
    proper_click_element,
    rozwin_ekstra_informacje_na_stronie_edycji_wydawnictwa,
)

from django_bpp.selenium_util import VERY_SHORT_WAIT_TIME, wait_for_page_load


@pytest.mark.parametrize("field_name", ["isbn", "e_isbn"])
def test_change_form_get_pbn_by_isbn_or_eisbn_via_api_pub_jest_w_api(
    admin_browser,
    asgi_live_server,
    field_name,
    pbn_serwer,
):
    url = reverse("admin:bpp_wydawnictwo_zwarte_add")
    with wait_for_page_load(admin_browser):
        admin_browser.visit(asgi_live_server.url + url)

    ROK = "2222"
    ISBN = "123"
    UID_REKORDU = "aosidjfoasidjfasdfghjksl"

    rozwin_ekstra_informacje_na_stronie_edycji_wydawnictwa(admin_browser)

    pub1 = pbn_publication_json(ROK, mongoId=UID_REKORDU, isbn=ISBN)
    pbn_serwer.expect_request(PBN_SEARCH_PUBLICATIONS_URL).respond_with_json(
        pbn_pageable_json([pub1])
    )
    pbn_serwer.expect_request(
        PBN_GET_PUBLICATION_BY_ID_URL.format(id=UID_REKORDU)
    ).respond_with_json(pub1)

    admin_browser.find_by_id("id_tytul_oryginalny").fill("nie istotny")
    admin_browser.find_by_id("id_rok").fill(ROK)
    admin_browser.find_by_id(f"id_{field_name}").fill(ISBN)

    if not admin_browser.find_by_id("id_isbn_pbn_get"):
        raise Exception("Nie mozna znalexc elementu")

    btn = admin_browser.find_by_id("id_isbn_pbn_get")
    proper_click_element(admin_browser, btn)

    time.sleep(VERY_SHORT_WAIT_TIME)

    admin_browser.find_by_css("input.select2-search__field").type(Keys.ENTER)
    time.sleep(VERY_SHORT_WAIT_TIME)

    assert admin_browser.find_by_id("id_pbn_uid").value == UID_REKORDU


@pytest.mark.parametrize("field_name", ["isbn", "e_isbn"])
def test_change_form_get_pbn_by_isbn_or_eisbn_via_api_pub_jest_w_lokalnej_bazie(
    admin_browser, asgi_live_server, field_name, pbn_serwer, transactional_db
):
    url = reverse("admin:bpp_wydawnictwo_zwarte_add")
    with wait_for_page_load(admin_browser):
        admin_browser.visit(asgi_live_server.url + url)

    ROK = "2222"
    ISBN = "123"
    UID_REKORDU = "aosidjfoasidjfasdfghjksl"

    try:
        pub = Publication.objects.create(
            mongoId=UID_REKORDU,
            verificationLevel="moze",
            verified=True,
            status=STATUS_ACTIVE,
            versions=[
                {
                    "current": True,
                    "object": {"year": ROK, "isbn": ISBN, "title": "Ten tego"},
                }
            ],
        )

        rozwin_ekstra_informacje_na_stronie_edycji_wydawnictwa(admin_browser)

        pbn_serwer.expect_request(PBN_SEARCH_PUBLICATIONS_URL).respond_with_json(
            pbn_pageable_json([])
        )

        admin_browser.find_by_id("id_tytul_oryginalny").fill("nie istotny")
        admin_browser.find_by_id("id_rok").fill(ROK)
        admin_browser.find_by_id(f"id_{field_name}").fill(ISBN)

        if not admin_browser.find_by_id("id_isbn_pbn_get"):
            raise Exception("Nie mozna znalexc elementu")

        btn = admin_browser.find_by_id("id_isbn_pbn_get")
        proper_click_element(admin_browser, btn)

        time.sleep(VERY_SHORT_WAIT_TIME)

        admin_browser.find_by_css("input.select2-search__field").type(Keys.ENTER)

        assert admin_browser.find_by_id("id_pbn_uid").value == UID_REKORDU
    finally:
        pub.delete()


@pytest.mark.parametrize("wydawnictwo", ["wydawnictwo_ciagle", "wydawnictwo_zwarte"])
def test_change_form_get_pbn_by_doi_via_api_jest_w_api(
    admin_browser, asgi_live_server, wydawnictwo, pbn_serwer
):
    url = reverse(f"admin:bpp_{wydawnictwo}_add")
    with wait_for_page_load(admin_browser):
        admin_browser.visit(asgi_live_server.url + url)

    ROK = "2222"
    DOI = "123"
    UID_REKORDU = "aosidjfoasidjfasdfghjksl"

    pub1 = pbn_publication_json(ROK, mongoId=UID_REKORDU, doi=DOI)
    pbn_serwer.expect_request(PBN_SEARCH_PUBLICATIONS_URL).respond_with_json(
        pbn_pageable_json([pub1])
    )
    pbn_serwer.expect_request(
        PBN_GET_PUBLICATION_BY_ID_URL.format(id=UID_REKORDU)
    ).respond_with_json(pub1)

    rozwin_ekstra_informacje_na_stronie_edycji_wydawnictwa(admin_browser)

    admin_browser.find_by_id("id_tytul_oryginalny").fill("nie istotny")
    admin_browser.find_by_id("id_rok").fill(ROK)
    admin_browser.find_by_id("id_doi").fill(DOI)

    if not admin_browser.find_by_id("id_doi_pbn_get"):
        raise Exception("Nie mozna znalexc elementu")

    btn = admin_browser.find_by_id("id_doi_pbn_get")
    proper_click_element(admin_browser, btn)

    time.sleep(VERY_SHORT_WAIT_TIME)

    with pytest.raises(NoAlertPresentException):
        admin_browser.driver.switch_to.alert

    admin_browser.find_by_css("input.select2-search__field").type(Keys.ENTER)

    assert admin_browser.find_by_id("id_pbn_uid").value == UID_REKORDU


@pytest.mark.parametrize("wydawnictwo", ["wydawnictwo_ciagle", "wydawnictwo_zwarte"])
def test_change_form_get_pbn_by_doi_via_api_nie_ma_w_api_jest_w_bazie(
    admin_browser, asgi_live_server, wydawnictwo, pbn_serwer, transactional_db
):
    url = reverse(f"admin:bpp_{wydawnictwo}_add")
    with wait_for_page_load(admin_browser):
        admin_browser.visit(asgi_live_server.url + url)

    ROK = "2222"
    DOI = "123"
    UID_REKORDU = "aosidjfoasidjfasdfghjksl"

    try:
        res = Publication.objects.create(
            mongoId=UID_REKORDU,
            verificationLevel="moze",
            verified=True,
            status="ACTIVE",
            versions=[
                {
                    "current": True,
                    "object": {"year": ROK, "doi": DOI, "title": "Ten tego"},
                }
            ],
        )
        pbn_serwer.expect_request(PBN_SEARCH_PUBLICATIONS_URL).respond_with_json(
            pbn_pageable_json([])
        )

        rozwin_ekstra_informacje_na_stronie_edycji_wydawnictwa(admin_browser)

        admin_browser.find_by_id("id_tytul_oryginalny").fill("nie istotny")
        admin_browser.find_by_id("id_rok").fill(ROK)
        admin_browser.find_by_id("id_doi").fill(DOI)

        if not admin_browser.find_by_id("id_doi_pbn_get"):
            raise Exception("Nie mozna znalexc elementu")

        btn = admin_browser.find_by_id("id_doi_pbn_get")
        proper_click_element(admin_browser, btn)

        time.sleep(VERY_SHORT_WAIT_TIME)

        with pytest.raises(NoAlertPresentException):
            admin_browser.driver.switch_to.alert

        time.sleep(VERY_SHORT_WAIT_TIME)

        admin_browser.find_by_css("input.select2-search__field").type(Keys.ENTER)
        time.sleep(VERY_SHORT_WAIT_TIME)

        assert admin_browser.find_by_id("id_pbn_uid").value == UID_REKORDU
    finally:
        res.delete()



# -*- encoding: utf-8 -*-
from django.db import models
from django.db.models import DO_NOTHING, QuerySet
from django_group_by.mixin import GroupByMixin

from bpp.models import ModelPunktowany, ModelZLiczbaCytowan, ModelZeStatusem


# Poniżej ważne jest to on_delete=DO_NOTHING, ponieważ bez tego Django
# będzie próbowało usuwać dane z tych tabel również, a te tabele to
# są VIEWs od strony SQLa, więc to się na ten moment nie uda (nie licząc
# tych VIEWs w PostgreSQL, które są modyfikowalne...)


class Nowe_SumyQuerySet(QuerySet, GroupByMixin):
    pass


class Nowe_Sumy_View(ModelZLiczbaCytowan, ModelPunktowany, models.Model):
    autor = models.OneToOneField("Autor", on_delete=DO_NOTHING)
    jednostka = models.OneToOneField("Jednostka", on_delete=DO_NOTHING)
    wydzial = models.ForeignKey("Wydzial", on_delete=DO_NOTHING)
    rok = models.IntegerField()

    afiliuje = models.BooleanField()

    status_korekty = models.ForeignKey("Status_Korekty", DO_NOTHING)

    objects = Nowe_SumyQuerySet.as_manager()

    class Meta:
        app_label = "bpp"
        managed = False
        unique_together = ("autor", "jednostka", "wydzial", "rok")


Sumy = Nowe_Sumy_View



import os

from django.conf import settings

PBN_CLIENT_DEFAULT_BASE_URL = "https://pbn-micro-alpha.opi.org.pl"


PBN_CLIENT_APP_ID = getattr(
    settings, "PBN_CLIENT_APP_ID", os.getenv("PBN_CLIENT_APP_ID")
)


PBN_CLIENT_APP_TOKEN = getattr(
    settings, "PBN_CLIENT_APP_TOKEN", os.getenv("PBN_CLIENT_APP_TOKEN")
)


PBN_CLIENT_BASE_URL = getattr(
    settings,
    "PBN_CLIENT_BASE_URL",
    os.getenv("PBN_CLIENT_BASE_URL", PBN_CLIENT_DEFAULT_BASE_URL),
)

PBN_CLIENT_USER_TOKEN = getattr(
    settings, "PBN_CLIENT_USER_TOKEN", os.getenv("PBN_CLIENT_USER_TOKEN")
)



from django_tables2 import Column


class DecimalColumn(Column):
    pass


class SummingColumn(Column):
    def render_footer(self, bound_column, table):
        return sum(bound_column.accessor.resolve(row) for row in table.data)



# Generated by Django 3.0.14 on 2021-07-13 20:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0013_oswiadczenieinstytucji_publikacjainstytucji"),
    ]

    operations = [
        migrations.AlterField(
            model_name="publikacjainstytucji",
            name="publicationVersion",
            field=models.UUIDField(blank=True, null=True),
        ),
    ]



from tempfile import NamedTemporaryFile

import openpyxl
import openpyxl.styles
from django.db import DEFAULT_DB_ALIAS, connections
from django_tables2.export import ExportMixin, TableExport
from openpyxl.utils import get_column_letter
from openpyxl.worksheet.filters import AutoFilter
from openpyxl.worksheet.table import Table, TableColumn, TableStyleInfo

from django.utils.itercompat import is_iterable


def drop_table(table_name, using=DEFAULT_DB_ALIAS):
    connection = connections[using]
    with connection.cursor() as cursor:
        cursor.execute("DROP TABLE IF EXISTS " + connection.ops.quote_name(table_name))


def _create_table_as(table_name, queryset, using=DEFAULT_DB_ALIAS, temporary=True):
    compiler = queryset.query.get_compiler(using=using)
    sql, params = compiler.as_sql()
    connection = connections[using]
    crt = "CREATE TABLE "
    if temporary:
        crt = "CREATE TEMPORARY TABLE "
    sql = crt + connection.ops.quote_name(table_name) + " AS " + sql
    drop_table(table_name, using=using)
    with connection.cursor() as cursor:
        cursor.execute(sql, params)


def create_temporary_table_as(table_name, queryset, using=DEFAULT_DB_ALIAS):
    return _create_table_as(table_name, queryset, using=using, temporary=True)


def create_table_as(table_name, queryset, using=DEFAULT_DB_ALIAS):
    return _create_table_as(table_name, queryset, using=using, temporary=False)


def insert_into(table_name, queryset, using=DEFAULT_DB_ALIAS):
    compiler = queryset.query.get_compiler(using=using)
    sql, params = compiler.as_sql()
    connection = connections[using]
    sql = "INSERT INTO " + connection.ops.quote_name(table_name) + " " + sql
    with connection.cursor() as cursor:
        cursor.execute(sql, params)


def clone_temporary_table(source_table, target_table, using=DEFAULT_DB_ALIAS):
    connection = connections[using]
    sql = (
        "CREATE TEMPORARY TABLE "
        + connection.ops.quote_name(target_table)
        + " AS SELECT * FROM "
        + connection.ops.quote_name(source_table)
    )
    drop_table(target_table, using=using)
    with connection.cursor() as cursor:
        cursor.execute(sql)


class MyTableExport(TableExport):
    """
    Fix https://github.com/jazzband/tablib/issues/252
    """

    FORMATS = {
        "xlsx": "application/vnd.ms-excel",
    }

    @classmethod
    def is_valid_format(self, export_format):
        if export_format not in ["xlsx", "pdf"]:
            return False
        return True

    def __init__(
        self, export_format, table, exclude_columns=None, export_description=None
    ):
        super().__init__(
            export_format=export_format, table=table, exclude_columns=exclude_columns
        )
        self.table = table
        self.export_description = export_description

    def export(self):
        return getattr(self, f"export_{self.format}")()

    def export_xlsx(self):

        wb = openpyxl.Workbook()
        ws = wb.active
        ws.title = "Sheet 1"

        table_name = "Table1"

        # Write the header row and make cells bold
        tablib_dataset = self.dataset

        if self.export_description:
            for elem in self.export_description:
                if is_iterable(elem):
                    ws.append(elem)
                    if len(elem) == 2:
                        ws[ws.max_row][0].font = openpyxl.styles.Font(bold=True)
                        ws[ws.max_row][0].alignment = openpyxl.styles.Alignment(
                            horizontal="right"
                        )
                        ws[ws.max_row][1].alignment = openpyxl.styles.Alignment(
                            horizontal="left"
                        )
                else:
                    ws.append([elem])

            ws.append([])

        ws.append(tablib_dataset.headers)

        table_columns = tuple(
            TableColumn(id=h, name=header)
            for h, header in enumerate(tablib_dataset.headers, start=1)
        )

        # Sumowana kolumna po stronie XLSX tworzona jest w taki sposób, że jeżeli jakakolwiek
        # kolumna tabeli ma footer, to jest tam wstawiana suma za pomocą funkcji =SUBTOTAL(9, ...)
        #
        # W przypadku gdyby to nie wystarczało w przyszłości, to do django_tables2.TableColumn
        # należałoby dopisać kod funkcji XLSa.
        #
        # Do tego, pierwsza kolumna (o indeksie zerowym) uzywana jest dla napisu "Suma"

        footer_row = []
        for no, elem in enumerate(self.table.columns):
            if no == 0:
                footer_row.append("Suma")
                total_column = table_columns[0]
                total_column.totalsRowLabel = footer_row[0]
                continue

            if elem.has_footer():
                count_column = table_columns[no]
                count_column.totalsRowFunction = "sum"
                footer_row.append(f"=SUBTOTAL(109,{table_name}[{elem.header}])")
                continue

            footer_row.append("")

        for cell in ws[ws.max_row : ws.max_row]:
            cell.font = openpyxl.styles.Font(bold=True)

        first_table_row = ws.max_row
        for row in tablib_dataset:
            ws.append(row)
        ws.append(footer_row)

        if tablib_dataset:
            max_column = ws.max_column
            max_column_letter = get_column_letter(max_column)
            max_row = ws.max_row

            style = TableStyleInfo(
                name="TableStyleMedium9",
                showFirstColumn=False,
                showLastColumn=False,
                showRowStripes=True,
                showColumnStripes=True,
            )
            tab = Table(
                displayName=table_name,
                ref=f"A{first_table_row}:{max_column_letter}{max_row }",
                autoFilter=AutoFilter(
                    ref=f"A{first_table_row}:{max_column_letter}{max_row - 1}"
                ),
                totalsRowShown=True,
                totalsRowCount=1,
                tableStyleInfo=style,
                tableColumns=table_columns,
            )

            ws.add_table(tab)

        max_width = 75
        for ncol, col in enumerate(ws.columns):
            max_length = 0
            column = col[0].column_letter  # Get the column name
            # Since Openpyxl 2.6, the column name is  ".column_letter" as .column became the column number (1-based)
            for cell in col:
                try:  # Necessary to avoid error on empty cells
                    if len(str(cell.value)) > max_length:
                        max_length = len(cell.value)
                except (ValueError, TypeError):
                    pass
            adjusted_width = (max_length + 2) * 1.1
            if adjusted_width > max_width:
                adjusted_width = max_width
            ws.column_dimensions[column].width = adjusted_width

        with NamedTemporaryFile() as tmp:
            wb.save(tmp.name)
            tmp.seek(0)
            return tmp.read()


class MyExportMixin(ExportMixin):
    def get_export_description(self):
        """Nadpisz tą funkcję, aby wygenerować pola opisowe na potrzeby XLS, np.
        'metkę' z parametrami raportu. Powinna zwrócić listę ciągów znaków, które zostaną
        wstawione przed tabelę, jeden pod drugim."""
        return

    def create_export(self, export_format):
        exporter = MyTableExport(
            export_format=export_format,
            table=self.get_table(**self.get_table_kwargs()),
            exclude_columns=self.exclude_columns,
            export_description=self.get_export_description(),
        )

        return exporter.response(filename=self.get_export_filename(export_format))


class InitialValuesFromGETMixin:
    def get_initial(self):
        initial = super().get_initial()
        if hasattr(self, "request"):
            for elem in self.get_form_class().base_fields.keys():
                value = self.request.GET.get(elem)
                if value is not None:
                    initial[elem] = value
        return initial



# Generated by Django 3.0.14 on 2021-07-13 20:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0014_auto_20210713_2229"),
    ]

    operations = [
        migrations.AlterField(
            model_name="publikacjainstytucji",
            name="publicationType",
            field=models.CharField(blank=True, max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name="publikacjainstytucji",
            name="publicationYear",
            field=models.PositiveSmallIntegerField(blank=True, null=True),
        ),
    ]



# Generated by Django 3.0.11 on 2021-03-15 20:04

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0249_dyscyplina_zrodla"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="dyscyplina_zrodla",
            options={
                "ordering": ["zrodlo__nazwa", "dyscyplina__nazwa"],
                "verbose_name": "przypisanie dyscypliny do źródła",
                "verbose_name_plural": "przypisanie dyscyplin do źródła",
            },
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0008_auto_20141126_1802'),
    ]

    operations = [
        migrations.AlterField(
            model_name='charakter_formalny',
            name='nazwa_w_primo',
            field=models.CharField(default=b'', choices=[(b'', b''), (b'Artyku\xc5\x82', b'Artyku\xc5\x82'), (b'Ksi\xc4\x85\xc5\xbcka', b'Ksi\xc4\x85\xc5\xbcka'), (b'Zas\xc3\xb3b tekstowy', b'Zas\xc3\xb3b tekstowy'), (b'Rozprawa naukowa', b'Rozprawa naukowa'), (b'Recenzja', b'Recenzja'), (b'Artyku\xc5\x82 prasowy', b'Artyku\xc5\x82 prasowy'), (b'Rozdzia\xc5\x82', b'Rozdzia\xc5\x82'), (b'Czasopismo', b'Czasopismo'), (b'Dane badawcze', b'Dane badawcze'), (b'Materia\xc5\x82 konferencyjny', b'Materia\xc5\x82 konferencyjny'), (b'Obraz', b'Obraz'), (b'Baza', b'Baza'), (b'Zestaw danych statystycznych', b'Zestaw danych statystycznych'), (b'Multimedia', b'Multimedia'), (b'Inny', b'Inny')], max_length=100, blank=True, help_text=b'\n    Nazwa charakteru formalnego w wyszukiwarce Primo, eksponowana przez OAI-PMH. W przypadku,\n    gdy to pole jest puste, prace o danym charakterze formalnym nie b\xc4\x99d\xc4\x85 udost\xc4\x99pniane przez\n    protok\xc3\xb3\xc5\x82 OAI-PMH.\n    ', verbose_name=b'Nazwa w Primo', db_index=True),
            preserve_default=True,
        ),
    ]



# Generated by Django 3.0.14 on 2021-08-18 21:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0029_indeksy_gist_inne_tabele"),
    ]

    operations = [
        migrations.AlterField(
            model_name="conference",
            name="verificationLevel",
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name="conference",
            name="verified",
            field=models.BooleanField(db_index=True, default=False),
        ),
        migrations.AlterField(
            model_name="institution",
            name="verificationLevel",
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name="institution",
            name="verified",
            field=models.BooleanField(db_index=True, default=False),
        ),
        migrations.AlterField(
            model_name="journal",
            name="verificationLevel",
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name="journal",
            name="verified",
            field=models.BooleanField(db_index=True, default=False),
        ),
        migrations.AlterField(
            model_name="publication",
            name="verificationLevel",
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name="publication",
            name="verified",
            field=models.BooleanField(db_index=True, default=False),
        ),
        migrations.AlterField(
            model_name="publisher",
            name="verificationLevel",
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name="publisher",
            name="verified",
            field=models.BooleanField(db_index=True, default=False),
        ),
        migrations.AlterField(
            model_name="scientist",
            name="verificationLevel",
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name="scientist",
            name="verified",
            field=models.BooleanField(db_index=True, default=False),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0071_fix_liczba_znakow_wyd'),
    ]

    operations = [
        migrations.RunSQL("drop index bpp_rekord_mat_search_index_idx"),
        migrations.RunSQL("CREATE INDEX bpp_rekord_mat_search_index_idx ON bpp_rekord_mat USING GIN(search_index)")
    ]



# -*- encoding: utf-8 -*-


from pbn_api.management.commands.util import PBNBaseCommand


class Command(PBNBaseCommand):
    def add_arguments(self, parser):
        super(Command, self).add_arguments(parser)
        parser.add_argument("command", nargs="+")

    def handle(
        self, app_id, app_token, base_url, user_token, command=None, *args, **options
    ):
        client = self.get_client(app_id, app_token, base_url, user_token)
        client.exec(command)



from django.views.generic import RedirectView

from django.contrib.contenttypes.models import ContentType


class NieistniejacaDyscyplina:
    pk = -1
    nazwa = "--"


class RedirectToAdmin(RedirectView):
    def get_redirect_url(self, *args, **kw):
        ctype = ContentType.objects.get(pk=self.kwargs["content_type_id"])
        return "/admin/%s/%s/%i/change/" % (
            ctype.app_label,
            ctype.model,
            self.kwargs["object_id"],
        )



from django.db import models, transaction
from django.db.models import JSONField

from ..exceptions import HttpException, StatementDeletionError
from .base import BasePBNMongoDBModel

from django.utils.functional import cached_property

from bpp.models import Uczelnia


class Institution(BasePBNMongoDBModel):
    class Meta:
        verbose_name = "Instytucja w PBN API"
        verbose_name_plural = "Instytucje w PBN API"

    name = models.TextField(null=True, blank=True, db_index=True)
    addressCity = models.TextField(null=True, blank=True, db_index=True)
    addressStreet = models.TextField(null=True, blank=True, db_index=True)
    addressStreetNumber = models.TextField(null=True, blank=True, db_index=True)
    addressPostalCode = models.TextField(null=True, blank=True, db_index=True)
    polonUid = models.TextField(null=True, blank=True, db_index=True)

    pull_up_on_save = [
        "name",
        "addressCity",
        "addressStreet",
        "addressStreetNumber",
        "addressPostalCode",
        "polonUid",
    ]

    def __str__(self):
        parent = self.value_or_none("object", "rootId")
        if parent:
            if parent == self.pk:
                parent = None
            else:
                try:
                    parent = Institution.objects.get(mongoId=parent)
                    parent = parent.name
                except Institution.DoesNotExist:
                    parent = None

        if parent is not None:
            parent = f" => {parent}"
        ret = (
            f"{self.name} {parent or ''}, "
            f"{self.addressCity or ''}, "
            f"{self.addressStreet or ''} {self.addressStreetNumber or ''} "
            f"{self.value_or_none('objects', 'email') or ''} "
            f"{self.value_or_none('objects', 'website') or ''} "
            f"(ID: {self.mongoId})"
        )
        while ret.find("  ") != -1:
            ret = ret.replace("  ", " ")
        ret = ret.replace(" ,", ",")
        while ret.find(",,") != -1:
            ret = ret.replace(",,", ",")
        ret = ret.replace(", (", " (")
        return ret

    @cached_property
    def rekord_w_bpp(self):
        from bpp.models import Jednostka

        try:
            return Jednostka.objects.get(pbn_uid_id=self.pk)
        except Jednostka.DoesNotExist:
            try:
                return Uczelnia.objects.get(pbn_uid_id=self.pk)
            except Uczelnia.DoesNotExist:
                pass


class PublikacjaInstytucji(models.Model):
    insPersonId = models.ForeignKey("pbn_api.Scientist", on_delete=models.CASCADE)
    institutionId = models.ForeignKey(Institution, on_delete=models.CASCADE)
    publicationId = models.ForeignKey("pbn_api.Publication", on_delete=models.CASCADE)
    publicationType = models.CharField(max_length=50, null=True, blank=True)
    userType = models.CharField(max_length=50, null=True, blank=True)
    publicationVersion = models.UUIDField(null=True, blank=True)
    publicationYear = models.PositiveSmallIntegerField(null=True, blank=True)
    snapshot = JSONField(null=True, blank=True)

    class Meta:
        verbose_name = "Publikacja instytucji"
        verbose_name_plural = "Publikacje instytucji"


class OswiadczenieInstytucji(models.Model):
    addedTimestamp = models.DateField()
    statedTimestamp = models.DateField(null=True, blank=True)
    area = models.PositiveSmallIntegerField()
    inOrcid = models.BooleanField()
    institutionId = models.ForeignKey(Institution, on_delete=models.CASCADE)
    personId = models.ForeignKey("pbn_api.Scientist", on_delete=models.CASCADE)
    publicationId = models.ForeignKey("pbn_api.Publication", on_delete=models.CASCADE)
    type = models.CharField(max_length=50)

    def get_bpp_publication(self):
        from bpp.models import (
            Praca_Doktorska,
            Praca_Habilitacyjna,
            Wydawnictwo_Ciagle,
            Wydawnictwo_Zwarte,
        )

        for klass in (
            Wydawnictwo_Ciagle,
            Wydawnictwo_Zwarte,
            Praca_Doktorska,
            Praca_Habilitacyjna,
        ):
            try:
                return klass.objects.get(pbn_uid_id=self.publicationId_id)
            except klass.MultipleObjectsReturned:
                ret = klass.objects.filter(pbn_uid_id=self.publicationId_id).first()
                print(
                    f"XXX DUPLIKAT! Dwie prace mają PBN UID {self.publicationId_id}, "
                    f"zwracam pierwszą {ret.tytul_oryginalny}!"
                )
            except klass.DoesNotExist:
                pass

    def get_bpp_autor(self):
        from bpp.models import Autor

        try:
            return Autor.objects.get(pbn_uid_id=self.personId_id)
        except Autor.DoesNotExist:
            return

    class Meta:
        verbose_name = "Oświadczenie instytucji"
        verbose_name_plural = "Oświadczenia instytucji"

    def sprobuj_skasowac_z_pbn(self, request=None, pbn_client=None):
        if pbn_client is None:
            uczelnia = Uczelnia.objects.get_for_request(request)
            if uczelnia is None:
                raise Uczelnia.DoesNotExist

            pbn_client = uczelnia.pbn_client(request.user.pbn_token)

        try:
            pbn_client.delete_publication_statement(
                self.publicationId_id, self.personId_id, self.type
            )
        except HttpException as e:
            raise StatementDeletionError(e.status_code, e.url, e.content)

    @transaction.atomic
    def delete(self, *args, **kw):
        # Jeżeli usunięte zostało jakiekolwiek oświadczenie to automatycznie dane SentData przestają
        # być aktualne, a system się na nich opiera. Zatem w tej sytuacji, kasujemy również
        # wysłane dane:
        from pbn_api.models import SentData

        SentData.objects.filter(pbn_uid_id=self.publicationId_id).delete(*args, **kw)
        return super().delete(*args, **kw)



from django.db import models


class LiczbaNField(models.DecimalField):
    def __init__(self, *args, **kw):
        if "max_digits" not in kw:
            kw["max_digits"] = 9

        if "decimal_places" not in kw:
            kw["decimal_places"] = 4

        super(LiczbaNField, self).__init__(*args, **kw)



# -*- encoding: utf-8 -*-

from bpp.tests import show_element

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

import pytest
from mock import Mock
from selenium.webdriver.support.expected_conditions import alert_is_present

from bpp.tests.util import assertPopupContains, scroll_into_view

from django_bpp.selenium_util import wait_for, wait_for_page_load

ID = "id_tytul_oryginalny"

pytestmark = [pytest.mark.slow, pytest.mark.selenium]


@pytest.fixture(scope="function")
def browser_z_wydawnictwem(admin_browser, asgi_live_server, wydawnictwo_ciagle):
    browser = admin_browser
    with wait_for_page_load(browser):
        browser.visit(
            asgi_live_server.url
            + reverse(
                "admin:bpp_wydawnictwo_ciagle_change", args=(wydawnictwo_ciagle.pk,)
            )
        )
    return browser


def test_admin_get_wos_information_clarivate_brak_danych(
    browser_z_wydawnictwem, denorms
):
    browser = browser_z_wydawnictwem
    elem = browser.find_by_id("id_liczba_cytowan_get")
    show_element(browser, elem)
    elem.click()

    def _():
        try:
            assertPopupContains(browser, "DOI")
            return True
        except AssertionError:
            pass

    wait_for(_)


def test_admin_get_wos_information_clarivate_pmid(
    uczelnia, mocker, browser_z_wydawnictwem
):
    from mock import Mock

    m = Mock()
    m.query_single = Mock(return_value={"timesCited": "31337"})
    mocker.patch("bpp.models.struktura.Uczelnia.wosclient", return_value=m)

    browser = browser_z_wydawnictwem
    browser.find_by_id("id_pubmed_id").type("31337")
    scroll_into_view(browser_z_wydawnictwem, "id_liczba_cytowan_get")
    browser.find_by_id("id_liczba_cytowan_get").click()
    wait_for(lambda: browser.find_by_id("id_liczba_cytowan_get").value == "Pobrano!")
    assert browser.find_by_id("id_liczba_cytowan_get").value == "Pobrano!"
    assert browser.find_by_id("id_liczba_cytowan").value == "31337"


def test_admin_get_wos_information_clarivate_err(
    uczelnia, browser_z_wydawnictwem, mocker
):
    m = Mock()
    m.query_single = Mock(side_effect=Exception("lel"))
    mocker.patch("bpp.models.struktura.Uczelnia.wosclient", return_value=m)

    browser = browser_z_wydawnictwem
    browser.find_by_id("id_pubmed_id").type("31337")
    scroll_into_view(browser_z_wydawnictwem, "id_liczba_cytowan_get")
    browser.find_by_id("id_liczba_cytowan_get").click()
    wait_for(lambda: alert_is_present()(browser.driver))
    assertPopupContains(browser, "lel")


def test_admin_get_wos_information_clarivate_misconfigured(
    uczelnia, browser_z_wydawnictwem, mocker
):
    uczelnia.clarivate_password = uczelnia.clarivate_username = ""
    uczelnia.save()

    browser = browser_z_wydawnictwem
    browser.find_by_id("id_pubmed_id").type("31337")
    scroll_into_view(browser_z_wydawnictwem, "id_liczba_cytowan_get")
    browser.find_by_id("id_liczba_cytowan_get").click()
    wait_for(lambda: alert_is_present()(browser.driver))
    assertPopupContains(browser, "Brak użytkownika API")

    uczelnia.clarivate_username = "fa"
    uczelnia.save()

    browser.find_by_id("id_liczba_cytowan_get").click()
    wait_for(lambda: alert_is_present()(browser.driver))
    assertPopupContains(browser, "Brak hasła do API")



import json

import pytest
from django.http.response import HttpResponseNotAllowed, HttpResponseRedirect
from django.urls import reverse
from model_bakery import baker

from django.contrib.contenttypes.models import ContentType

from bpp.models.autor import Autor
from bpp.models.cache import Rekord
from bpp.models.patent import Patent
from bpp.models.praca_doktorska import Praca_Doktorska
from bpp.models.praca_habilitacyjna import Praca_Habilitacyjna
from bpp.models.profile import BppUser
from bpp.models.struktura import Jednostka
from bpp.models.wydawnictwo_ciagle import Wydawnictwo_Ciagle
from bpp.models.wydawnictwo_zwarte import Wydawnictwo_Zwarte
from bpp.models.zrodlo import Zrodlo
from bpp.views.global_nav import global_nav_redir


def test_global_nav_redir_err():
    class FakeRequest:
        method = "POST"
        GET = {}

    res = global_nav_redir(FakeRequest(), "")
    assert isinstance(res, HttpResponseNotAllowed)


@pytest.mark.parametrize(
    ["model", "source"],
    [
        (Autor, "user"),
        (Rekord, "user"),
        (Jednostka, "user"),
        (Zrodlo, "user"),
        (Autor, "admin"),
        (Wydawnictwo_Ciagle, "admin"),
        (Wydawnictwo_Zwarte, "admin"),
        (Zrodlo, "admin"),
        (Patent, "admin"),
        (Praca_Habilitacyjna, "admin"),
        (Praca_Doktorska, "admin"),
        (BppUser, "admin"),
    ],
)
@pytest.mark.django_db
def test_global_nav_redir(model, source, typy_odpowiedzialnosci):
    class FakeRequest:
        method = "GET"
        GET = {"source": source}

    if model == Rekord:
        baker.make(Wydawnictwo_Ciagle)
        a = Rekord.objects.first()
    else:
        a = baker.make(model)

    res = global_nav_redir(
        FakeRequest(), f"{ContentType.objects.get_for_model(a).pk}-{a.pk}"
    )

    assert isinstance(res, HttpResponseRedirect)


def test_global_nav_ukrywanie_statusow_przed_korekta_praca_schowana(
    client, uczelnia, wydawnictwo_ciagle, przed_korekta, po_korekcie
):
    wydawnictwo_ciagle.status_korekty = przed_korekta
    wydawnictwo_ciagle.tytul_oryginalny = "123 test"
    wydawnictwo_ciagle.save()

    uczelnia.ukryj_status_korekty_set.create(status_korekty=przed_korekta)

    res = client.get(reverse("bpp:navigation-autocomplete") + "?q=123")

    assert len(json.loads(res.content)["results"]) == 0


def test_global_nav_ukrywanie_statusow_przed_korekta_praca_schowana_ale_dla_admina_widoczna(
    admin_client, uczelnia, wydawnictwo_ciagle, przed_korekta, po_korekcie
):
    wydawnictwo_ciagle.status_korekty = przed_korekta
    wydawnictwo_ciagle.tytul_oryginalny = "123 test"
    wydawnictwo_ciagle.save()

    uczelnia.ukryj_status_korekty_set.create(status_korekty=przed_korekta)

    res = admin_client.get(reverse("bpp:navigation-autocomplete") + "?q=123")

    assert len(json.loads(res.content)["results"]) == 1


def test_global_nav_ukrywanie_statusow_przed_korekta_praca_widoczna(
    client, uczelnia, wydawnictwo_ciagle, przed_korekta, po_korekcie
):
    wydawnictwo_ciagle.status_korekty = po_korekcie
    wydawnictwo_ciagle.tytul_oryginalny = "123 test"
    wydawnictwo_ciagle.save()

    uczelnia.ukryj_status_korekty_set.create(status_korekty=przed_korekta)

    res = client.get(reverse("bpp:navigation-autocomplete") + "?q=123")

    assert len(json.loads(res.content)["results"]) == 1



from django.db import models

# Ta klasa na obecną chwilę nie zawiera nic, ale używamy jej, aby oznaczyć
# odpowiednio pola, gdzie chodzi o rok:
YearField = models.IntegerField


class DOIField(models.CharField):
    def __init__(self, *args, **kw):
        if "help_text" not in kw:
            kw["help_text"] = "Digital Object Identifier (DOI)"

        if "max_length" not in kw:
            kw["max_length"] = 2048

        super().__init__(*args, **kw)



from datetime import datetime

import pytest
from model_bakery import baker
from multiseek import logic
from multiseek.logic import DIFFERENT, EQUAL, AutocompleteQueryObject

from bpp import const
from bpp.models import (
    Autor_Dyscyplina,
    Dyscyplina_Naukowa,
    Typ_Odpowiedzialnosci,
    Zewnetrzna_Baza_Danych,
)
from bpp.models.autor import Autor
from bpp.models.cache import Rekord
from bpp.models.openaccess import (
    Czas_Udostepnienia_OpenAccess,
    Licencja_OpenAccess,
    Wersja_Tekstu_OpenAccess,
)
from bpp.multiseek_registry import (
    EQUAL_PLUS_SUB_FEMALE,
    EQUAL_PLUS_SUB_UNION_FEMALE,
    UNION,
    AktualnaJednostkaAutoraQueryObject,
    CharakterOgolnyQueryObject,
    DataUtworzeniaQueryObject,
    DOIQueryObject,
    DyscyplinaQueryObject,
    ForeignKeyDescribeMixin,
    JednostkaQueryObject,
    LicencjaOpenAccessUstawionaQueryObject,
    LiczbaAutorowQueryObject,
    NazwiskoIImieQueryObject,
    ObcaJednostkaQueryObject,
    OpenaccessCzasPublikacjiQueryObject,
    OpenaccessLicencjaQueryObject,
    OpenaccessWersjaTekstuQueryObject,
    OstatnieNazwiskoIImie,
    OstatnioZmieniony,
    PierwszaJednostkaQueryObject,
    PierwszeNazwiskoIImie,
    PierwszyWydzialQueryObject,
    PublicDostepDniaQueryObject,
    RodzajKonferenckjiQueryObject,
    SlowaKluczoweQueryObject,
    StronaWWWUstawionaQueryObject,
    Typ_OdpowiedzialnosciQueryObject,
    TypOgolnyAutorQueryObject,
    TypOgolnyRecenzentQueryObject,
    TypOgolnyRedaktorQueryObject,
    TypOgolnyTlumaczQueryObject,
    TytulPracyQueryObject,
    WydzialQueryObject,
    ZewnetrznaBazaDanychQueryObject,
)


@pytest.mark.django_db
@pytest.mark.parametrize("value", ["foo", "bar", "łódź", "jeszcze test"])
@pytest.mark.parametrize(
    "operation",
    [
        logic.CONTAINS,
        logic.NOT_CONTAINS,
        logic.STARTS_WITH,
        logic.NOT_STARTS_WITH,
        logic.EQUAL,
    ],
)
def test_TytulPracyQueryObject(value, operation):
    ret = TytulPracyQueryObject(
        field_name="tytul_oryginalny", label="Tytuł oryginalny", public=True
    ).real_query(value, operation)
    assert Rekord.objects.filter(*(ret,)).count() == 0


@pytest.mark.django_db
@pytest.mark.parametrize("tested_field", ["www", "public_www"])
def test_StronaWWWUstawionaQueryObject_jedna_z(wydawnictwo_zwarte, tested_field):
    fields = ["public_www", "www"]
    for field in fields:
        setattr(wydawnictwo_zwarte, field, None)
    wydawnictwo_zwarte.save()

    qry = StronaWWWUstawionaQueryObject("x1x").real_query(True, logic.EQUAL_FEMALE)
    assert Rekord.objects.filter(*(qry,)).count() == 0

    qry = StronaWWWUstawionaQueryObject("x1x").real_query(False, logic.EQUAL_FEMALE)
    assert Rekord.objects.filter(*(qry,)).count() == 1

    setattr(wydawnictwo_zwarte, tested_field, "http://onet.pl/")
    wydawnictwo_zwarte.save()

    qry = StronaWWWUstawionaQueryObject("x1x").real_query(True, logic.EQUAL_FEMALE)
    assert Rekord.objects.filter(*(qry,)).count() == 1

    qry = StronaWWWUstawionaQueryObject("x1x").real_query(False, logic.EQUAL_FEMALE)
    assert Rekord.objects.filter(*(qry,)).count() == 0


@pytest.mark.django_db
def test_StronaWWWUstawionaQueryObject_obydwie_strony(
    wydawnictwo_zwarte,
):
    fields = ["public_www", "www"]
    for field in fields:
        setattr(wydawnictwo_zwarte, field, "http://onet.pl/")
    wydawnictwo_zwarte.save()

    qry = StronaWWWUstawionaQueryObject("x1x").real_query(True, logic.EQUAL_FEMALE)
    assert Rekord.objects.filter(*(qry,)).count() == 1


@pytest.mark.django_db
def test_multiseek_licencja_openaccess_ustawiona(wydawnictwo_zwarte):
    lqo = LicencjaOpenAccessUstawionaQueryObject(
        field_name="licencja_openaccess", label="X", public=True
    )
    res = lqo.real_query(True, logic.EQUAL)
    assert Rekord.objects.filter(*(res,)).count() == 0

    res = lqo.real_query(False, logic.EQUAL)
    assert Rekord.objects.filter(*(res,)).count() == 1


@pytest.mark.django_db
def test_multiseek_charakter_formalny_ogolny(wydawnictwo_zwarte, ksiazka_polska):
    wydawnictwo_zwarte.charakter_formalny = ksiazka_polska
    wydawnictwo_zwarte.save()

    lqo = CharakterOgolnyQueryObject(field_name="charakter", label="X")
    res = lqo.real_query("książka", logic.EQUAL)
    assert Rekord.objects.filter(*(res,)).count() == 1

    res = lqo.real_query("książka", logic.DIFFERENT)
    assert Rekord.objects.filter(*(res,)).count() == 0


def test_DataUtworzeniaQueryObject():
    d = DataUtworzeniaQueryObject()
    assert d.value_for_description('[""]') == str(datetime.now().date())
    assert d.value_for_description('["2018-01-01 12:00:00"]') == "2018-01-01"
    assert (
        d.value_for_description('["2018-01-01 12:00:00", "2018-01-01 12:00:00"]')
        == "od 2018-01-01 do 2018-01-01"
    )


@pytest.mark.django_db
def test_DyscyplinaQueryObject(autor_jan_kowalski, wydawnictwo_zwarte, rok, jednostka):
    dn = baker.make(Dyscyplina_Naukowa)
    Autor_Dyscyplina.objects.create(
        autor=autor_jan_kowalski, dyscyplina_naukowa=dn, rok=rok
    )
    wydawnictwo_zwarte.dodaj_autora(
        autor_jan_kowalski, jednostka, dyscyplina_naukowa=dn
    )

    res = DyscyplinaQueryObject().real_query(dn, EQUAL)
    assert Rekord.objects.filter(res).count() == 1

    res = DyscyplinaQueryObject().real_query(dn, DIFFERENT)
    assert Rekord.objects.filter(res).count() == 0


@pytest.mark.django_db
@pytest.mark.parametrize(
    "klass,model",
    [
        (OpenaccessWersjaTekstuQueryObject, Wersja_Tekstu_OpenAccess),
        (OpenaccessLicencjaQueryObject, Licencja_OpenAccess),
        (OpenaccessCzasPublikacjiQueryObject, Czas_Udostepnienia_OpenAccess),
    ],
)
def test_multiseek_openaccess(klass, model, openaccess_data):
    f = model.objects.all().first()
    x = klass().value_from_web(f.nazwa)
    assert f == x


@pytest.mark.django_db
def test_ForeignKeyDescribeMixin_value_for_description():
    class Tst(ForeignKeyDescribeMixin, AutocompleteQueryObject):
        field_name = "foo"
        model = Autor

    x = Tst()
    assert x.value_for_description("123").find("został usunięty") > 0


def test_NazwiskoIImieQueryObject(autor_jan_nowak):
    n = NazwiskoIImieQueryObject()

    ret = n.real_query(autor_jan_nowak, logic.EQUAL)
    assert ret is not None

    ret = n.real_query(autor_jan_nowak, logic.DIFFERENT)
    assert ret is not None

    ret = n.real_query(autor_jan_nowak, UNION)
    assert ret is not None

    ret = n.real_query(None, logic.EQUAL)
    assert ret is not None

    ret = n.real_query(None, logic.DIFFERENT)
    assert ret is not None

    ret = n.real_query(None, UNION)
    assert ret is not None


def test_JednostkaQueryObject(jednostka):
    n = JednostkaQueryObject()

    ret = n.real_query(jednostka, logic.EQUAL)
    assert ret is not None

    ret = n.real_query(jednostka, logic.DIFFERENT)
    assert ret is not None

    ret = n.real_query(jednostka, UNION)
    assert ret is not None

    ret = n.real_query(None, logic.EQUAL)
    assert ret is not None

    ret = n.real_query(None, logic.DIFFERENT)
    assert ret is not None

    ret = n.real_query(None, UNION)
    assert ret is not None

    ret = n.real_query(jednostka, EQUAL_PLUS_SUB_FEMALE)
    assert ret is not None

    ret = n.real_query(jednostka, EQUAL_PLUS_SUB_UNION_FEMALE)
    assert ret is not None


def test_WydzialQueryObject(wydzial):
    n = WydzialQueryObject()

    ret = n.real_query(wydzial, logic.EQUAL)
    Rekord.objects.filter(ret)

    ret = n.real_query(wydzial, logic.DIFFERENT)
    Rekord.objects.filter(ret)

    ret = n.real_query(wydzial, UNION)
    Rekord.objects.filter(ret)

    ret = n.real_query(None, logic.EQUAL)
    Rekord.objects.filter(ret)

    ret = n.real_query(None, logic.DIFFERENT)
    Rekord.objects.filter(ret)

    ret = n.real_query(None, UNION)
    Rekord.objects.filter(ret)


def test_PierwszyWydzialQueryObject(wydzial):
    n = PierwszyWydzialQueryObject()

    ret = n.real_query(wydzial, logic.EQUAL)
    Rekord.objects.filter(ret)

    ret = n.real_query(wydzial, logic.DIFFERENT)
    Rekord.objects.filter(ret)

    ret = n.real_query(wydzial, UNION)
    Rekord.objects.filter(ret)

    ret = n.real_query(None, logic.EQUAL)
    Rekord.objects.filter(ret)

    ret = n.real_query(None, logic.DIFFERENT)
    Rekord.objects.filter(ret)

    ret = n.real_query(None, UNION)
    Rekord.objects.filter(ret)


@pytest.mark.django_db
@pytest.mark.parametrize("logic_arg", [logic.EQUAL, UNION])
def test_PierwszeNazwiskoIImie_real_query(
    wydawnictwo_zwarte, autor_jan_kowalski, jednostka, logic_arg
):
    wydawnictwo_zwarte.dodaj_autora(autor_jan_kowalski, jednostka)

    r = Rekord.objects.filter(
        PierwszeNazwiskoIImie().real_query(autor_jan_kowalski, logic_arg)
    )

    assert len(r) == 1


@pytest.mark.django_db
@pytest.mark.parametrize(
    "logic_arg",
    [logic.EQUAL, UNION, EQUAL_PLUS_SUB_FEMALE, EQUAL_PLUS_SUB_UNION_FEMALE],
)
def test_PierwszaJednostka_realQuery(
    wydawnictwo_zwarte, autor_jan_kowalski, jednostka, logic_arg
):
    wydawnictwo_zwarte.dodaj_autora(autor_jan_kowalski, jednostka)

    r = Rekord.objects.filter(
        PierwszaJednostkaQueryObject().real_query(jednostka, logic_arg)
    )

    assert len(r) == 1


@pytest.mark.django_db
@pytest.mark.parametrize("logic_arg", [logic.EQUAL, UNION])
def test_PierwszeNazwiskoIImie_real_query_2(logic_arg):
    r = Rekord.objects.filter(PierwszeNazwiskoIImie().real_query(None, logic_arg))

    assert len(r) == 0


@pytest.mark.django_db
def test_Typ_OdpowiedzialnosciQueryObject():
    t = baker.make(Typ_Odpowiedzialnosci, typ_ogolny=const.TO_AUTOR)

    res = Typ_OdpowiedzialnosciQueryObject().real_query(t, logic.DIFFERENT)
    assert Rekord.objects.filter(res).count() == 0

    res = Typ_OdpowiedzialnosciQueryObject().real_query(t, logic.DIFFERENT)
    assert Rekord.objects.filter(res).count() == 0

    res = Typ_OdpowiedzialnosciQueryObject().real_query(None, logic.DIFFERENT)
    assert Rekord.objects.filter(res).count() == 0

    res = Typ_OdpowiedzialnosciQueryObject().real_query(None, UNION)
    assert Rekord.objects.filter(res).count() == 0


@pytest.mark.django_db
def test_TypOgolnyAutorQueryObject(autor_jan_nowak):
    baker.make(Typ_Odpowiedzialnosci, typ_ogolny=const.TO_AUTOR)

    res = TypOgolnyAutorQueryObject().real_query(autor_jan_nowak, logic.DIFFERENT)
    assert Rekord.objects.filter(res).count() == 0

    res = TypOgolnyAutorQueryObject().real_query(autor_jan_nowak, UNION)
    assert Rekord.objects.filter(res).count() == 0

    res = TypOgolnyAutorQueryObject().real_query(None, logic.DIFFERENT)
    assert Rekord.objects.filter(res).count() == 0

    res = TypOgolnyAutorQueryObject().real_query(None, UNION)
    assert Rekord.objects.filter(res).count() == 0


@pytest.mark.django_db
def test_TypOgolnyRedaktorQueryObject(autor_jan_nowak):
    baker.make(Typ_Odpowiedzialnosci, typ_ogolny=const.TO_REDAKTOR)

    res = TypOgolnyRedaktorQueryObject().real_query(autor_jan_nowak, logic.DIFFERENT)
    assert Rekord.objects.filter(res).count() == 0

    res = TypOgolnyRedaktorQueryObject().real_query(autor_jan_nowak, UNION)
    assert Rekord.objects.filter(res).count() == 0


@pytest.mark.django_db
def test_TypOgolnyTlumaczQueryObject(autor_jan_nowak):
    baker.make(Typ_Odpowiedzialnosci, typ_ogolny=const.TO_TLUMACZ)

    res = TypOgolnyTlumaczQueryObject().real_query(autor_jan_nowak, logic.DIFFERENT)
    assert Rekord.objects.filter(res).count() == 0

    res = TypOgolnyTlumaczQueryObject().real_query(autor_jan_nowak, UNION)
    assert Rekord.objects.filter(res).count() == 0


@pytest.mark.django_db
def test_TypOgolnyRecenzentQueryObject(autor_jan_nowak):
    baker.make(Typ_Odpowiedzialnosci, typ_ogolny=const.TO_RECENZENT)

    res = TypOgolnyRecenzentQueryObject().real_query(autor_jan_nowak, logic.DIFFERENT)
    assert Rekord.objects.filter(res).count() == 0

    res = TypOgolnyRecenzentQueryObject().real_query(autor_jan_nowak, UNION)
    assert Rekord.objects.filter(res).count() == 0


@pytest.mark.django_db
def test_OstatnieNazwiskoIImie(autor_jan_nowak):
    res = OstatnieNazwiskoIImie().real_query(autor_jan_nowak, logic.DIFFERENT)
    assert Rekord.objects.filter(res).count() == 0


@pytest.mark.django_db
def test_OstatnioZmieniony():
    res = OstatnioZmieniony().real_query(datetime.now(), logic.EQUAL)
    assert Rekord.objects.filter(res).count() == 0


@pytest.mark.django_db
def test_RodzajKonferenckjiQueryObject():
    res = RodzajKonferenckjiQueryObject().real_query("krajowa", logic.DIFFERENT)
    assert Rekord.objects.filter(res).count() == 0


@pytest.mark.django_db
def test_LiczbaAutorowQueryObject():
    res = LiczbaAutorowQueryObject().real_query(5, logic.DIFFERENT)
    assert Rekord.objects.filter(res).count() == 0


@pytest.mark.django_db
def test_ZewnetrznaBazaDanychQueryObject():
    zbd = baker.make(Zewnetrzna_Baza_Danych)
    res = ZewnetrznaBazaDanychQueryObject().real_query(zbd, logic.DIFFERENT)
    assert Rekord.objects.filter(res).count() == 0


@pytest.mark.django_db
def test_DOIQueryObject():
    res = DOIQueryObject().real_query("foo", logic.EQUAL)
    assert Rekord.objects.filter(res).count() == 0


@pytest.mark.django_db
def test_DostepDniaQueryObject():
    res = PublicDostepDniaQueryObject().real_query(True, logic.EQUAL)
    assert Rekord.objects.filter(res).count() == 0


@pytest.mark.django_db
def test_ObcaJednostkaQueryObject(
    wydawnictwo_zwarte,
    autor_jan_kowalski,
    obca_jednostka,
):
    wydawnictwo_zwarte.dodaj_autora(autor_jan_kowalski, obca_jednostka, afiliuje=False)

    res = ObcaJednostkaQueryObject().real_query(True, logic.EQUAL)
    assert Rekord.objects.filter(res).count() == 1


@pytest.mark.django_db
def test_SlowaKluczoweQueryObject(wydawnictwo_zwarte):
    wydawnictwo_zwarte.slowa_kluczowe.add("foo")
    wydawnictwo_zwarte.save()

    res = SlowaKluczoweQueryObject().real_query("foo", logic.EQUAL)
    assert Rekord.objects.filter(res).count() == 1


@pytest.mark.django_db
@pytest.mark.parametrize(
    "param",
    [
        logic.EQUAL,
        logic.DIFFERENT,
        EQUAL_PLUS_SUB_FEMALE,
    ],
)
def test_AktualnaJednostkaAutoraQueryObject(jednostka, param):
    res = AktualnaJednostkaAutoraQueryObject().real_query(jednostka, param)
    assert res is not None



from django.conf.urls import url


def bad_page(request):
    1 / 0


urlpatterns = [
    url(r'test_500', bad_page, name="test_500"),

    # Trzeba to dodać, inaczej rendering strony się wysypie
    url(r'noop', bad_page, name="password_change")
]



import pytest

from bpp.util import knapsack, strip_html, wytnij_isbn_z_uwag


@pytest.mark.parametrize(
    "input,output,rest",
    [
        ("ISBN 978-83-7374-091-4, ten tego", "978-83-7374-091-4", "ten tego"),
        ("ISBN 83-200-1817-X", "83-200-1817-X", ""),
        ("tu nie ma ISBNu", None, None),
        ("ISBN-10 978-83-7374-091-4; ISBN-13 958498498894, ten tego", None, None),
    ],
)
def test_wytnij_isbn_z_uwag(input, output, rest):
    res = wytnij_isbn_z_uwag(input)
    if output is None:
        assert res is None
        return

    isbn, reszta = res
    assert isbn == output
    assert reszta == rest


def test_knapsack_empty():
    assert knapsack(10, [], [], []) == (0, [])


@pytest.mark.timeout(5)
def test_knapsack_very_long():
    # Test na optymalizacje sytuacji gdy chcemy dostac znacznie wiecej punktow niz wychdozi z danej listy,
    knapsack(99900, [1] * 99800, [1] * 99800, [1] * 99800)


def test_knapsack_zwracaj_liste():
    assert knapsack(
        10,
        [1, 2, 3, 4, 5],
        [1, 2, 3, 4, 5],
        [
            "a",
            "b",
            "c",
            "d",
            "e",
        ],
        True,
    ) == (10, ["d", "c", "b", "a"])


def test_knapsack_nie_zwracaj_listy():
    assert knapsack(
        10,
        [1, 2, 3, 4, 5],
        [1, 2, 3, 4, 5],
        [
            "a",
            "b",
            "c",
            "d",
            "e",
        ],
        zwracaj_liste_przedmiotow=False,
    ) == (10, [])


@pytest.mark.django_db
def test_ModelZOpisemBibliograficznym(wydawnictwo_ciagle):
    assert wydawnictwo_ciagle.opis_bibliograficzny() != ""


@pytest.mark.parametrize(
    "i,o", [("<b>test", "test"), ("<b>test</b>", "test"), ("", ""), (None, None)]
)
def test_strip_html(i, o):
    assert strip_html(i) == o



from typing import Union

from pathspec.util import normalize_file


def remove_trailing_interpunction(s: str) -> str:
    try:
        while s[-1] in ".,-":
            s = s[:-1]
    except IndexError:
        return ""

    return s


def remove_extra_spaces(s: str) -> str:
    while s.find("  ") >= 0:
        s = s.replace("  ", " ")
    s = s.strip()
    return s


def normalize_first_name(s: str) -> str | None:
    if not isinstance(s, str) or s is None or s == "" or not s:
        return
    return remove_extra_spaces(s)


normalize_last_name = normalize_first_name
normalize_publisher = normalize_first_name


def normalize_nullboleanfield(s: Union[str, None, bool]) -> Union[bool, None]:
    if isinstance(s, bool):
        return s
    if s is None:
        return
    s = s.strip().lower()

    if s in ["true", "tak", "prawda", "t", "p"]:
        return True
    if s in ["false", "nie", "fałsz", "falsz", "f", "n"]:
        return False


def normalize_skrot(s):
    if s is None:
        return
    return remove_extra_spaces(s.lower().replace(" .", ". "))


def normalize_tytul_naukowy(s):
    return normalize_skrot(s)


def normalize_title(s: str) -> str | None:
    if s is None or not s:
        return
    while s.find("  ") >= 0:
        s = s.replace("  ", " ")
    return s.strip()


ONLINE_STR = "[online]"


def normalize_tytul_publikacji(s):
    if s is None:
        return
    ret = remove_trailing_interpunction(normalize_skrot(s))
    if ret.endswith(ONLINE_STR):
        ret = ret[: -len(ONLINE_STR)].strip()
    ret = ret.replace("\r", " ").replace("\n", " ")
    while ret.find("  ") >= 0:
        ret = ret.replace("  ", " ")
    return ret


def normalize_funkcja_autora(s: str) -> str:
    return normalize_skrot(s).lower()


def normalize_grupa_pracownicza(s: str):
    return normalize_skrot(s)


def normalize_wymiar_etatu(s: str):
    return normalize_skrot(s)


def normalize_nazwa_jednostki(s: str) -> str:
    return remove_extra_spaces(s.strip())


def normalize_nazwa_wydawcy(s: str) -> str:
    return remove_extra_spaces(s.strip())


def normalize_exception_reason(s: str) -> str:
    return remove_extra_spaces(s.replace("\n", " ").replace("\r\n", " "))


def normalize_tytul_zrodla(s):
    return normalize_skrot(s)


def normalize_nazwa_dyscypliny(s):
    return normalize_skrot(s)


def normalize_isbn(isbn: str) -> str | None:
    if isbn is None or not isinstance(isbn, str) or not isbn:
        return

    return isbn.replace(".", "").replace("-", "").replace(" ", "").strip()


normalize_issn = normalize_isbn


def normalize_kod_dyscypliny(k):
    if k is None:
        return None

    if k.endswith("_0"):
        k = k[:-2]
        return f"{k[0]}.{int(k[1:])}"
    if k.find(".") >= 0:
        return k
    return f"{k[0]}.{int(k[1:])}"


def normalize_public_uri(public_uri):
    if public_uri is not None:
        return public_uri.strip()


def normalize_doi(s: str) -> None | str:
    """
    https://www.doi.org/doi_handbook/2_Numbering.html#2.4
    """

    if s is None:
        return

    s = s.strip()

    if not s:
        return

    return (
        s.lower()
        .replace("http://", "")
        .replace("https://", "")
        .replace("dx.doi.org/", "")
        .replace("doi.org/", "")
    )


def normalize_filename(s: str) -> str:
    s = normalize_file(s).replace(" ", "_").replace(",", "_")

    while s.find("__") != -1:
        s = s.replace("__", "_")
    return s


def normalize_orcid(s: str) -> str | None:
    if not isinstance(s, str) or s is None or s == "" or not s:
        return
    return (
        s.strip()
        .lower()
        .replace("http://orcid.org/", "")
        .replace("https://orcid.org/", "")
    )



# Generated by Django 3.2.15 on 2022-10-02 20:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("dynamic_columns", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="modeladmincolumn",
            options={"ordering": ("parent", "ordering")},
        ),
        migrations.AddField(
            model_name="modeladmincolumn",
            name="enabled",
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name="modeladmincolumn",
            name="ordering",
            field=models.PositiveSmallIntegerField(default=1),
            preserve_default=False,
        ),
        migrations.DeleteModel(
            name="ModelAdminSetting",
        ),
    ]



# Generated by Django 3.0.11 on 2021-02-28 12:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taggit", "0003_taggeditem_add_unique_index"),
        ("bpp", "0238_drop_pesel_md5"),
    ]

    operations = [
        migrations.AddField(
            model_name="autor",
            name="pbn_uuid",
            field=models.UUIDField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="autor",
            name="system_kadrowy_id",
            field=models.PositiveSmallIntegerField(
                blank=True,
                help_text="Identyfikator cyfrowy, używany do matchowania autora z danymi z systemu kadrowego Uczelni",
                null=True,
                verbose_name="Identyfikator w systemie kadrowym",
            ),
        ),
        migrations.AddField(
            model_name="autor_jednostka",
            name="podstawowe_miejsce_pracy",
            field=models.NullBooleanField(),
        ),
    ]



# Generated by Django 3.0.14 on 2021-11-09 23:00

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0308_ewaluacja_upowaznienia_view"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="wydawnictwo_ciagle_streszczenie",
            options={
                "verbose_name": "streszczenie wydawnictwa ciągłego",
                "verbose_name_plural": "streszczenia wydawnictw ciągłych",
            },
        ),
        migrations.AlterModelOptions(
            name="wydawnictwo_zwarte_streszczenie",
            options={
                "verbose_name": "streszczenie wydawnictwa zwartego",
                "verbose_name_plural": "streszczenia wydawnictw zwatrtych",
            },
        ),
        migrations.AlterField(
            model_name="wydawnictwo_ciagle_streszczenie",
            name="rekord",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="streszczenia",
                to="bpp.Wydawnictwo_Ciagle",
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_zwarte_streszczenie",
            name="rekord",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="streszczenia",
                to="bpp.Wydawnictwo_Zwarte",
            ),
        ),
    ]



# -*- encoding: utf-8 -*-
from optparse import make_option
import sys

from django.core.management import BaseCommand
from django.db import transaction

from bpp.models.cache import Rekord
from bpp.models import Autor, Autorzy

class Command(BaseCommand):
    help = 'Kasuje autorow bez prac'

    @transaction.atomic
    def handle(self, *args, **options):
        for a in Autor.objects.all().only("id"):
            res = Autorzy.objects.filter(autor_id=a.pk)[:1]
            if res.count() == 0:
                print("BRAK PRAC: ID=%s, autor=%s, USUWAM!" % (a.pk, str(a)))
                a.delete()


import arrow
from django.db import transaction

from pbn_api.management.commands.util import PBNBaseCommand

from bpp.models import Jezyk, Rekord
from bpp.util import pbar


class Command(PBNBaseCommand):
    """Nadpisuje wszystkie dyscypliny dla zrodel z odpowiednikami z PBNu"""

    @transaction.atomic
    def handle(self, verbosity=1, *args, **options):
        for rekord in pbar(Rekord.objects.exclude(pbn_uid_id=None)):
            original = rekord.original
            if original.streszczenia.exists():
                continue

            publication = original.pbn_uid
            versions = sorted(
                publication.versions,
                key=lambda obj: arrow.get(obj["createdTime"]),
                reverse=True,
            )
            for elem in versions:
                abstracts = elem.get("object").get("abstracts")
                if abstracts:
                    for lang_code, text in abstracts.items():
                        try:
                            jezyk_streszczenia = Jezyk.objects.get(pbn_uid_id=lang_code)
                        except Jezyk.DoesNotExist:
                            print(
                                f"\n\nNIE zaimportowano streszczenia - brak takiego jezyka w systemie jak {lang_code}"
                            )
                            continue

                        original.streszczenia.create(
                            jezyk_streszczenia=jezyk_streszczenia, streszczenie=text
                        )

                    print(
                        f"Zaimportowano {len(abstracts)} streszczen dla rekordu {original} "
                        f"ze snapshotu {elem['createdTime']}"
                    )
                    break



from denorm import denormalized, depend_on_related
from django.db import models
from django.db.models import CASCADE, SET_NULL

from .autor import Autor
from .struktura import Jednostka
from .system import Typ_Odpowiedzialnosci
from .wydawnictwo_zwarte import Wydawnictwo_Zwarte_Baza

from django.contrib.postgres.fields import ArrayField

from django.utils.functional import cached_property

from bpp.models import (
    Charakter_Formalny,
    ModelZOplataZaPublikacje,
    ModelZPBN_UID,
    NieMaProcentowMixin,
)
from bpp.models.abstract import DwaTytuly


class Praca_Doktorska_Baza(
    NieMaProcentowMixin,
    ModelZPBN_UID,
    ModelZOplataZaPublikacje,
    Wydawnictwo_Zwarte_Baza,
):

    jednostka = models.ForeignKey(Jednostka, CASCADE)

    @property
    def autorzy_set(self):
        class FakeAutorDoktoratuHabilitacji:
            autor = self.autor
            jednostka = self.jednostka
            zapisany_jako = f"{ autor.nazwisko or '' } { autor.imiona or '' }"

        class FakeSet(list):
            def all(self):
                return self

            def select_related(self, *args, **kw):
                return self

            def odpiete_dyscypliny(self, *args, **kw):
                return FakeSet([])

        ret = FakeAutorDoktoratuHabilitacji()
        ret.typ_odpowiedzialnosci = Typ_Odpowiedzialnosci.objects.get(skrot="aut.")
        return FakeSet([ret])

    def autorzy_dla_opisu(self):
        return self.autorzy_set

    class Meta:
        abstract = True

    #
    # Cache framework by django-denorm-iplweb
    #

    denorm_always_skip = ("ostatnio_zmieniony",)

    @denormalized(models.TextField, default="")
    @depend_on_related(
        "bpp.Autor",
        foreign_key="autor",
        only=("nazwisko", "imiona", "tytul_id"),
    )
    @depend_on_related("bpp.Jednostka", only=("nazwa", "skrot", "wydzial_id"))
    @depend_on_related("bpp.Status_Korekty")
    def opis_bibliograficzny_cache(self):
        return self.opis_bibliograficzny()

    @denormalized(ArrayField, base_field=models.TextField(), blank=True, null=True)
    @depend_on_related(
        "bpp.Autor",
        foreign_key="autor",
        only=(
            "nazwisko",
            "imiona",
        ),
    )
    def opis_bibliograficzny_autorzy_cache(self):
        return [f"{self.autor.nazwisko} {self.autor.imiona}"]

    @denormalized(models.TextField, blank=True, null=True)
    @depend_on_related(
        "bpp.Autor",
        foreign_key="autor",
        only=(
            "nazwisko",
            "imiona",
        ),
    )
    def opis_bibliograficzny_zapisani_autorzy_cache(self):
        return f"{self.autor.nazwisko} {self.autor.imiona}"

    @denormalized(
        models.SlugField,
        max_length=400,
        unique=True,
        db_index=True,
        null=True,
        blank=True,
    )
    @depend_on_related(
        "bpp.Autor",
        foreign_key="autor",
        only=("nazwisko", "imiona"),
    )
    def slug(self):
        return self.get_slug()


class _Praca_Doktorska_PropertyCache:
    @cached_property
    def charakter_formalny(self):
        return Charakter_Formalny.objects.get(skrot="D")


_Praca_Doktorska_PropertyCache = _Praca_Doktorska_PropertyCache()


class Praca_Doktorska(Praca_Doktorska_Baza):
    autor = models.ForeignKey(Autor, CASCADE)

    promotor = models.ForeignKey(
        Autor, SET_NULL, related_name="promotor_doktoratu", blank=True, null=True
    )

    @cached_property
    def charakter_formalny(self):
        return _Praca_Doktorska_PropertyCache.charakter_formalny

    class Meta:
        verbose_name = "praca doktorska"
        verbose_name_plural = "prace doktorskie"
        app_label = "bpp"
        ordering = ("rok", "tytul_oryginalny")

    def clean(self):
        DwaTytuly.clean(self)
        ModelZOplataZaPublikacje.clean(self)



from django.apps import AppConfig
from django.conf import settings
from django.db.models.signals import post_migrate


def create_entries(sender, **kw):
    from .views.autor import WyborOsoby
    from .views.ewaluacja import ParametryRaportSlotowEwaluacja
    from .views.uczelnia import UtworzRaportSlotowUczelnia

    for elem in (
        WyborOsoby,
        ParametryRaportSlotowEwaluacja,
        UtworzRaportSlotowUczelnia,
    ):
        elem().get_initial()


class RaportSlotowConfig(AppConfig):
    name = "raport_slotow"

    def ready(self):
        if getattr(settings, "TESTING"):
            return
        post_migrate.connect(create_entries, sender=self)
